CHART = {};

/**
 * Colors used for the plot lines - palette originally from CFD
 */
CHART.colors = ["#B9264F", "#2966B8", "#FF800D", "#59955C", "#9A03FE", "#FF2626", "#FF5353","#C8B400", "#23819C"];

CHART.dayTickFormatter = function(val, axis) {
	var legendEntry = this.legendData[val];
	return legendEntry ? legendEntry.date : "";
}

/**
 * Returns the flot defaults
 */
CHART.getPlotDefaults = function(isWallboard, data) {
	// fetch the chart legend if available
	var legend;
	if (data && data.legend) {
		legend = data.legend;
	}
	
	if (isWallboard) {
		/*
		 * Wallboard graph flot defaults
		 */
		return {
			xaxis: {
		    },
		    yaxis: {
		        min:0
		    },
		    series: {
		        lines: { show: true, lineWidth: 9 },
		        points: { show: false },
		        shadowSize: 0
		    },
		    legend: {
		        position: "ne"
		    },
		    grid: {
		    	noGridBottomOffsetDelta: 0,
		        show: false
		    }
		};
	} else {
		/*
		 * Default graph flot defaults
		 */
		return {
			xaxis: {
				tickFormatter: CHART.dayTickFormatter,
				legendData: legend
		    },
            yaxis: {
                min:0
	        },
		    series: {
		        lines: {
                    show: true,
                    lineWidth: 5
                },
		        points: {
                    show: false
                },
		        shadowSize: 0
		    },
		    legend: { },
            grid: {
                color: "#333", // primary color used for outline and labels
                tickColor: "#eee", // color used for the ticks                
                borderWidth: 1, // in pixels
                borderColor: "#999" // set if different from the grid color
            }
		};
	}
};

/**
 * Returns the burndown relevant curves, adapted to be displayed on the wallbaord 
 */
CHART.burndownPlotData = function(data) {
	var plotData = [];
	if (!data || !data.charts) {
		return plotData;
	}
	
	// each chart contains a burndown and guideline curve - that's the only ones we are interested in
	var guidelines = [];
	var burndowns = [];
	var burndownIndex = 0;
	AJS.$.each(data.charts, function(index, chart) {
		var contextName = chart.context.name;
		var burndownFound = false;
		AJS.$.each(chart.curves, function(i, curve) {
			if (curve.type == "GUIDE") {
				var plot = {
					points: { show: false },
					lines: { show: true, lineWidth: 2 },
					shadowSize: 0,
					color: "#333333",
					data: CHART.convertPoints(curve.points)
				}
				guidelines.push(plot);
			} else if (curve.type == "BURNDOWN" && !burndownFound) {
				var plot = {
		            color: CHART.colors[(burndownIndex++ % CHART.colors.length)],
		            label: contextName,
		            data: CHART.convertPoints(curve.points)
		        };
				burndowns.push(plot);
				burndownFound = true;
			}
		});
	});
	
	// add the plot data for the guidelines first, then for the burndown lines
	AJS.$.each(guidelines, function(index, plot) {
		plotData.push(plot);
	});
	AJS.$.each(burndowns, function(index, plot) {
		plotData.push(plot);
	});
	return plotData;
};

/**
 * Transforms the plotData colors
 */
CHART.rewritePlotDataColors = function(plotData) {
	AJS.$(plotData).each(function(i, plot) {
		plot.color = CHART.colors[(i % CHART.colors.length)];
	});
};

/**
 * Returns the plot data for the charts provided as-is, which means that all
 * curves will be added exactly as provided by the charts.
 * 
 * Normally this only makes sense if only a single chart is available
 */
CHART.identityPlotData = function(data) {
	var plotData = [];
	if (!data || !data.charts) {
		return plotData;
	}
	
	AJS.$.each(data.charts, function(i, chart) {
		AJS.$.each(chart.curves, function(i, curve) {
			var plot = {
				color: curve.color,
	            label: curve.name,
	            data: CHART.convertPoints(curve.points)
	        };
	        plotData.push(plot);
        });
	});
	return plotData;
};

/**
 * Converts the returned data values into what flot expects
 */
CHART.convertPoints = function(points) {
	var data = [];
	for (var j = 0, count = points.length; j < count; j++)
    {
        var newPoint = [points[j].x, points[j].y];
        data[j] = newPoint;
    }
	return data;
};

/**
 * Chart template function
 *
 *	<div class="gg-wrapper"> 
 *		<canvas></canvas> 
 *		<div class="gg-legend-wrap"> 
 *			<ul class="gg-legend"> 
 *				<li><span style="background: rgb(0,0,0);"></span>Name 1</li> 
 *				<li><span></span>Name 2</li> 
 *				<li><span></span>Name 3</li>	
 *			</ul> 
 *		</div> 
 *	</div>
 *	 
 */
CHART.template = function (gadget, data, baseUrl, chartClass, isWallboard, plotDefaults, plotData, titleElements) {
	gadget.getView().empty(); // .css("display", "block");

	var $body = AJS.$('body');
	$body.addClass(chartClass);

	// handle the case where no data is available to display
	if (!data || !data.charts || data.charts.length<1 || !plotData || plotData.length<1) {
		var html = ''
	        + '<div class="gg-wrapper">'
	        + '  <p class="gg-error">'
	        + gadget.getMsg("gh.gadget.common.chart.nodata")
			+ '  </p>'
	        + '</div>';
		gadget.getView().append(AJS.$(html));
		return;
	}
	
	var html = ''
        + '<div class="gg-wrapper">'
        + '  <div class="gg-chart-group">'
		+ '    <div id="canvas-0">&nbsp</div>'
		+ '    <div class="gg-legend-wrap"></div>'
		+ '  </div>'
        + '</div>';
	gadget.getView().append(AJS.$(html));

	// add the title
	var $ggWrapper = AJS.$('.gg-wrapper', gadget.getView());
	GH.gadget.utils.insertHeaderElements(titleElements, $ggWrapper);

	// make sure we hide the loading
	gadget.hideLoading();

	// fetch the graph element
	var $graphElem = AJS.$('#canvas-0');
	var $legendElem = AJS.$('div.gg-legend-wrap', gadget.getView());

	// make sure the view is visible, otherwise $graphElem has size 0x0 and the plot will fail (happens after closing the configuration screen)
	AJS.$(".view").css("display", "block");

	// calculate the size of the graph
	// set the size of the canvas to 4:3
	var ratio = 4 / 3;
	var width = $graphElem.width();
	var height = width / ratio;

	// current wallbaord layout implementation assigns a fixed height to the iframes,
	// which we will have to honor. Check whether the iframe holds a size, and if so
	// calculate a relative size to it
	if (isWallboard) {
		var $parent = $body.parent().parent();
		var parentHeight = $parent.height();
		if (parentHeight < height)
		{
			var titleHeight = AJS.$('h2', $ggWrapper).height();
			if (titleHeight > 0) titleHeight += 15;
			height = parentHeight - 10 - titleHeight; // -border around graph, - header height
		}
	}
	$graphElem.height(height);
	
	// fetch the data and construct the options for the plot
	var plotOptions = AJS.$.extend({}, plotDefaults);
	plotOptions.xaxis.min = data.startX;
	plotOptions.xaxis.max = data.endX; // plotInfo.maxSprintEndDate;
	plotOptions.legend.container = $legendElem;
	
    // now plot the graph
    AJS.$.plot($graphElem, plotData, plotOptions);

    // remove block display after rendering
    AJS.$(".view").css("display", "");

    // rewrite the legend to our own markup
    var $legendUl = AJS.$('<ul class="gg-legend" />');
    var $rows = $legendElem.find('tr');
    $rows.each(function(index, elem) {
    	var $elem = AJS.$(elem);
    	var label = $elem.find('td.legendLabel').text();
    	var color = $elem.find('td.legendColorBox > div > div').css('border-top-color');
    	$legendUl.append(AJS.$('<li><span style="background:'+color+'"></span>' + label + '</li>'));
    });
    $legendElem.children().remove();
    $legendElem.append($legendUl);
};
