/**
 * Card field edit handler.
 * 
 * Delegates actual work to field specific objects.
 */
GH.CardField = {
        
    /**
     * Holds the controllers for the different field types.
     * key: controller type
     * value: controller object
     */
    controllers: {},
        
    /**
     * Get the field parameters for a given dom element.
     */
    getEditFieldRequestParams: function(fieldInfo) {
        var fieldId = fieldInfo.fieldId;
        var issueKey = AJS.$(fieldInfo.elem).closest('.gh-issue-wrapper, .gh-step-parent, .js-issue-wrapper').attr('data-issuekey');
        
        // find the issue
        var issue = findIssueByKey(issueKey);
        if(! issue) return;
        
        var issueId = issue.id;
        var stepId = issue.board.id;

        var params = {
            key: issueKey,
            id: issueId,
            stepId: stepId,
            fieldId: fieldId
        };
        return params;
    },
    
    /**
     * Gathers the field data necessary to edit a field
     */
    getFieldInfo: function(elem) {
        // fetch the field that should be edited
        var field = AJS.$(elem).closest('.gh-issue-field');
        var fieldId = field.attr('data-fieldid');
        if (!fieldId) return;
        var inputType = field.attr('data-inputtype');
        if (!inputType) return;
        
        // fetch the controller for that given type
        var controller = GH.CardField.controllers[inputType];
        if (!controller) return;
        
        return {
            elem: field,
            fieldId: fieldId,
            inputType: inputType,
            controller: controller
        };
    },
    
    /**
     * Handles field click events. This function is triggered through live events
     */
    editFieldClickHandler : function(event) {
        // fetch the field data object, containing all infos we need for an edit
        var fieldInfo = GH.CardField.getFieldInfo(this);
        
        if (fieldInfo) {
            GH.CardField.editField(fieldInfo);
        } else {
            return true;
        }
    },
    
    /**
     * Edit a field.
     */ 
    editField : function(fieldInfo) {
        // fetch the field params
        var params = GH.CardField.getEditFieldRequestParams(fieldInfo);
        if (!params) return;
        
        // perform an ajax request to get the edit field values
        Boards.fillMissingParams(params);
        var url = Boards.ctx + "/secure/" + (Boards.isTaskBoard()?"TB":"") +"EditField" + ".jspa" ;
        AJS.$.ajax({
            type: "GET",
            url: url,
            data: params,
            dataType: 'json',
            success: function(editFieldData) {
                // make sure we stop all edits on the to be edited issue prior to continue
                var issue = findIssueByKey(params.key);
                if (issue) {
                    issue.cancelEdit();
                }
                
                // let the controller handle the response
                fieldInfo.controller.editField(fieldInfo, editFieldData);
            }/*,
            complete: function(xhr) { console.log(xhr.status, xhr.responseText); }*/
        });
    },
    
    /** Submits a new field value. */
    submitFieldValue : function(fieldInfo, newValue) {
        // fetch the parameters to submit
        var params = GH.CardField.getEditFieldRequestParams(fieldInfo);
        params.fieldValue = newValue;
        
        // perform ajax call
        Boards.fillMissingParams(params);
        var url = Boards.ctx + "/secure/" + (Boards.isTaskBoard()?"TB":"") +"UpdateFieldJson" + ".jspa" ; 
        
        AJS.$.ajax({
        	type: "POST",
            url: url,
            data: params,
            dataType: "json",
            success: function(response) {
                if (response.success) {
                    GH.CardField.submitFieldSuccessResponseHandler(fieldInfo, response.success);
                }
                else {
                    GH.CardField.submitFieldErrorResponseHandler(fieldInfo, response.error);
                }
            }/*,
            complete: function(xhr) { console.log(xhr.status, xhr.responseText); }*/
        });
    },
    
    /**
     * Handles the update field error response.
     */
    submitFieldErrorResponseHandler: function(fieldInfo, error) {
        // search mode - uses an alert
        if (Boards.inSearchMode) {
            var message = '';
            AJS.$(error.errors).each(function(index, error) {
                message += error.fieldname + ": " + error.message + "\n";
            });
            alert(message);
            
            // controller callback
            fieldInfo.controller.cancelEdit(fieldInfo);
        }
        // normal error, display a popup
        else {
            //cancel edit, then display the dialog
            
            // TODO: does this even make sense in this case?
            if (error.issueKeysToCancel) {
                Boards.getBoard('$selectedBoard.id').cancel(error.issueKeysToCancel);
            }
            
            // cancel the field
            fieldInfo.controller.cancelEdit(fieldInfo);
            
            // then display an error dialog
            var messageDom = GH.CardField.renderErrors(error.errors);
            Boards.dynAJSPopup(400, 200);
            Boards.AJSpopup.addHeader(error.i18n['gh.error.errortitle']);
            AJS.$('.aui-dialog').addClass('gh-dialog gh-dialog-field-error');
            Boards.AJSpopup.addPanel("", "panel1");
            Boards.AJSpopup.getCurrentPanel().html(messageDom.html()); // yes, makes a lot of sense
            Boards.AJSpopup.addCloseAction(error.i18n['gh.boards.close'], error.modifierKey);
            Boards.showAJSPopup(0, 0);
        }
    },

    /** Transforms a list of errors into a dom element, */
    renderErrors: function(errors) {
        var errorsDiv = AJS.$('<div><ul class="gh-error"></ul></div>');
        AJS.$(errors).each(function(index, error) {

            var errorFieldName = AJS.$('<strong />').text(error.fieldname + ': ');
            var errorMessage = AJS.$('<li />').text(error.message);
            AJS.$(errorMessage).prepend(errorFieldName);
            AJS.$(errorsDiv).find('ul').append(errorMessage);

        });
        return errorsDiv;
    },
    
    /**
     * Handles update field responses.
     * The response object contains the new values for the edited field as well as additional update information.
     */
    submitFieldSuccessResponseHandler: function(fieldInfo, response) {
        // search board
        if (Boards.inSearchMode) {
            Boards.needsRefresh = true;
            if (response.refreshStats && AJS.$('#search-column').size()) {
                var searchKey = AJS.$('#searchKey').val();
                var searchType = AJS.$('#searchType').val();
                Boards.refreshSearchStats(searchKey, searchType);
            }
            GH.CardField.updateField(response.issueKey, response.fieldToUpdate);
            Boards.returnToSearch();
        }
        // task board
        else if (Boards.is('TaskBoard')) {
            // refresh board, this could be just the column and stats of that column
            if (response.refreshBoard) {
                // lets just reload the board for now
                Boards.getBoardForIssue(response.issueKey);
            }
            // refresh the column and its stats
            else if (response.refreshStats) {
                var issue = findIssueByKey(response.issueKey);
                issue.board.refreshStepBoard();
                if (response.refreshStats) {
                    Boards.updater('gh-step-head', 'RefreshHeader', $H(), true, Boards.doneWorking);
                }
            }
            // just update the issue
            else {
                GH.CardField.updateField(response.issueKey, response.fieldToUpdate);
            }
        }
        // planning board
        else {
            // refresh board as well as the column
            if (response.refreshBoard) {
                // load the board, try to select the updated issue
                Boards.mainBoard.refreshMainBoard(Boards.mainBoard.getStart(), [ response.issueKey ]);
                Boards.refreshColumn();
            }
            // only refresh card and possible the column, markers
            else {
                GH.CardField.updateField(response.issueKey, response.fieldToUpdate);
                if (response.refreshStats) {
                    Boards.refreshColumn();
                    Boards.mainBoard.refreshMarkers();
                }
            }
        }
    },
    
    /**
     * Updates a card field with new values
     */
    updateField: function(issueKey, fieldData) {
        // find the controller for the given field
        var controller = GH.CardField.controllers[fieldData.inputType]; 
        if (!controller) return;
        
        // find the issue
        var issues = findIssuesByKey(issueKey);
        
        AJS.$(issues).each(function(idx, issue) {
			// locate whether we got that specific field rendered
			var fields = AJS.$('#' + issue.cKey).find('div[data-fieldid=' + fieldData.fieldId + ']');
			fields = AJS.$(fields); // @todo this line is MR's jquery learning
			fields.each(function(index, field) {
			    controller.updateField(field, fieldData);
			});

			// refresh the parent issue
			issue.refreshParent();
        });
    }
};

// bind a live event to catch clicks on the issues
AJS.$(document).delegate('.gh-issue-field .gh-editable', 'click', GH.CardField.editFieldClickHandler);
