/**
 * This library provides AJAX methods with defaults and automatic error handling.
 */
GH.Ajax = {};

GH.Ajax.CONTEXT_PATH = '${contextPath}';
GH.Ajax.REST_URL_BASE = '/rest/greenhopper/1.0';

/**
 * Holds xhr requests information, such as any ongoing xhr
 * and timestamp of last execution
 */
GH.Ajax.xhrRegistery = {};
GH.Ajax.stallTimeout = 300; // stall time duration for subsequent requests

/**
 * Get the xhr data for a given resource id. An empty object is returned if no data exists yet.
 */
GH.Ajax.getXhrData = function(resourceId) {
    var reqData = { timestamp: 0 /*, xhr: null*/ };
    if (resourceId) {
        if (GH.Ajax.xhrRegistery[resourceId]) {
            reqData = GH.Ajax.xhrRegistery[resourceId];
        } else {
            // store the new reqData in the registry
            reqData = GH.Ajax.xhrRegistery[resourceId] = reqData;
        }
    }
    return reqData;
};

/**
 * Make a GET request. Setting a different HTTP method will be overridden.
 *
 * @param settings : the jQuery settings object (see jQuery.ajax), with GH extensions:
 */
GH.Ajax.get = function(settings, resourceId) {
    settings = settings || {};
    var originalSuccessHandler = settings.success;

    // fetch the request registry for the given resourceId
    var reqData = GH.Ajax.getXhrData(resourceId);

    // enrich the ajax settings object with a custom success handler, that proxies errors to the appropriate callback
    AJS.$.extend(settings, {
        type : 'GET',
        success : function(result, textStatus, xhr) {
            // request successfully coompleted, remove xhr object from registry
            if (reqData.xhr) {
                // clear out request, no need to cancel anymore
                delete reqData.xhr;
            }

            result = GH.Ajax.extendResult(result);
            if (result && result.isSuccess() && originalSuccessHandler) {
                // it was a successful call, so forward it to the original handler
                originalSuccessHandler.call(this, result.success, textStatus, xhr);

            } else {
                // not successful, so there should be errors in here.
                GH.Ajax.handleContextualErrors(result, settings.errorContextMap, settings.contextualErrorRenderer);
                GH.Ajax.handleGlobalErrors(result, settings.onGlobalError);
            }
        }
    });

    //if we don't have an error handler defined then generate one.
    settings.error = GH.Ajax._generateErrorHandler(settings.onGlobalError);

    // aboard previous request if still ongoing
    if (reqData.xhr) {
        // stop ongoing request
        AJS.log('aborting request for ' + resourceId);
        reqData.xhr.abortMarker = true;
        reqData.xhr.abort();
        delete reqData.xhr;
    }

    // execute new request
    GH.Ajax.executeGet(settings, resourceId);
};

/** Executes the passed get request - or stalls in case this is a subsequent request of the same type. */
GH.Ajax.executeGet = function(settings, resourceId) {
    // update the request timestamp
    var reqData = GH.Ajax.getXhrData(resourceId);
    var lastRequestStamp = reqData.timestamp;
    reqData.timestamp = new Date().getTime();

    // check whether we should execute now or stall
    if (reqData.timestamp - lastRequestStamp < GH.Ajax.stallTimeout) {
        AJS.log('stalling request for ' + resourceId + ', time since last request: ' + (reqData.timestamp - lastRequestStamp));
        // clear previous timeout, therefore discard the previous request settings
        if(reqData.timeoutId) {
            window.clearTimeout(reqData.timeoutId);
        }
        var callback = function() {
            GH.Ajax.executeGet(settings, resourceId);
        };
        reqData.timeoutId = window.setTimeout(callback, GH.Ajax.stallTimeout);
        return;
    }

    // execute request
    AJS.log('executing request for ' + resourceId);
    reqData.xhr = JIRA.SmartAjax.makeRequest(settings);
};

/**
 * Make a POST request. Setting a different HTTP method will be overridden. The datatype returned by the server is expected to be JSON.
 *
 * @param settings : the jQuery settings object (see jQuery.ajax)
 *        GH extensions are:
 *      - errorContextMap : map between contextIds as returned from the server and a CSS selector defining where the field errors should be displayed
 *                          Alternatively, specify a function callback as value that receives the error and does custom stuff with it.
 *      - contextualErrorRenderer : optional. Alternative renderer for contextual errors. Takes the error as parameter and has to return the rendered view.
 *      - onGlobalError : optional. Global error callback
 */
GH.Ajax.post = function(settings) {
    // POST is vulnerable to XSRF, so we use JSON as transfer encoding - this isn't possible using plain HTML forms.
    var jsonData = JSON.stringify(settings.data); 
    AJS.$.extend(settings, { 
        type : 'POST',
        contentType : 'application/json',
        data : jsonData
    });
    GH.Ajax.store(settings);
};

/**
 * Make a PUT request. Setting a different HTTP method will be overridden. The datatype returned by the server is expected to be JSON.
 * 
 * @param settings : the jQuery settings object (see jQuery.ajax)
 *        GH extensions are:
 *      - errorContextMap : map between contextIds as returned from the server and a CSS selector defining where the field errors should be displayed.
 *                          Alternatively, specify a function callback as value that receives the error and does custom stuff with it.
 *      - contextualErrorRenderer : optional. Alternative renderer for contextual errors. Takes the error as parameter and has to return the rendered view.
 *      - onGlobalError : optional. Global error callback
 */
GH.Ajax.put = function(settings) {
    AJS.$.extend(settings, { type : 'PUT' });
    GH.Ajax.store(settings);
};

/**
 * Make a DELETE request. Setting a different HTTP method will be overridden. The datatype returned by the server is expected to be JSON.
 *
 * @param settings : the jQuery settings object (see jQuery.ajax)
 *        GH extensions are::
 *      - errorContextMap : map between contextIds as returned from the server and a CSS selector defining where the field errors should be displayed.
 *                          Alternatively, specify a function callback as value that receives the error and does custom stuff with it.
 *      - contextualErrorRenderer : optional. Alternative renderer for contextual errors. Takes the error as parameter and has to return the rendered view.
 *      - onGlobalError : optional. Global error callback
 */
GH.Ajax.del = function(settings) {
    AJS.$.extend(settings, { type : 'DELETE' });
    GH.Ajax.store(settings);
};

/**
 * Called to make an ajax request that stores data. Overrides datatype and handles a result of type ResultEntry (see Java class for details). 
 */
GH.Ajax.store = function(settings) {
    settings = settings || {};
    
    // remember the original success handler, because we're going to intercept that
    var originalSuccessHandler = settings.success;

    // override some settings
    AJS.$.extend(true, settings, {
        dataType : 'json',
        success : function(result, textStatus, xhr) {
            // extend the result with some convenience methods
            result = GH.Ajax.extendResult(result);

            // see if result was successful
            if (result.isSuccess() && originalSuccessHandler) {
                // it was a successful call, so forward it to the original handler
                originalSuccessHandler.call(this, result, textStatus, xhr);

            } else {
                // not successful, so there should be errors in here.
                GH.Ajax.handleContextualErrors(result, settings.errorContextMap, settings.contextualErrorRenderer);
                GH.Ajax.handleGlobalErrors(result, settings.onGlobalError);
            }
        }
    });
    //if we don't have an error handler defined then generate one.
    settings.error = GH.Ajax._generateErrorHandler(settings.onGlobalError);

    JIRA.SmartAjax.makeRequest(settings);
};

/**
 * Pick up the field errors returned by the server, resolve the respective element in the page where they should be 
 * displayed, and render there errors there.
 * 
 * @param res : the result, extended through GH.Ajax.extendResult
 * @param errorContextMap : map between the internal field name (as added to ErrorCollection) and the CSS selector for where
 *          it should be displayed (or a function that does the rendering).
 * @param renderer : optional. Used to render the individual error messages.
 * 
 * @returns true if there were field errors to be handled, false otherwise.
 */
GH.Ajax.handleContextualErrors = function(res, errorContextMap, renderer) {
    errorContextMap = errorContextMap || {}; 
    
    // check if we have anything useful to work on
    if (!res.hasContextualErrors()) return false;
    
    // render the contextual errors
    AJS.$(res.getContextualErrors()).each(function(index, ctxError) {
        
        // look up the context. This can be a jQuery selector, a DOM element or a function.
        var ctx = errorContextMap[ctxError.contextId];
        
        if (!ctx) {
            // the server specified a context, but the client didn't map it, so we render the error as global.
            // the global error handling comes after this, and since we don't want to mess with the AUI message
            // directly (there might be actual global errors as well), we'll just add it to the data object.
            AJS.$(ctxError.errors).each(function(index, err) {
                res.addGlobalError(err);                
            });
            
        } else if (AJS.$.isFunction(ctx)) {
            // the mapped context is a function, so pass in the error and let it do whatever it wants to do with it.
            ctx(ctxError, res);
            
        } else {
            // we have a context, and it's not a function, so it's either an element or a CSS selector. Regardless, we JQ it
            // and show the errors next to it.
            ctx = AJS.$(ctx);
            
            // remove previous errors
            ctx.siblings(".ghx-error").remove();

            // and attach the new ones.
            var renderedError;
            if (renderer) {
                // caller specified an own renderer
                renderedError = renderer(ctxError);
                
            } else {
                // default renderer
                renderedError = GH.tpl.rapid.notification.renderContextualErrors({ 
                    errors : ctxError.errors 
                });
            }
            
            // add as siblings to the context element
            ctx.after(renderedError);
        }
    });
    
    return true;
};

/**
 * Pick up the global errors returned by the server and display them through the notification system.
 * Global errors can be either produced by the server or added by the contextual error handler for unmapped
 * contextual errors.
 * 
 * @param res : the result object (see ResultEntry), extended by GH.Ajax.extendResult
 * @param onGlobalError : optional. will be called after errors have been displayed, and have the wrapped result passed in.
 */
GH.Ajax.handleGlobalErrors = function(res, onGlobalError) {
    
    // see if we have something to work with.
    if (!res.hasGlobalErrors()) return false;
    
    // display errors with an AUI message
    GH.RapidBoard.Notification.showErrors({ errors : res.getGlobalErrors()});

    if(onGlobalError)
    {
        onGlobalError(res);
    }

    return true;
};

/**
 * Turn the given uri into a default REST url. 
 * Example: passing in '/rapidView' will result in '/rest/greenhopper/1.0/rapidView'.
 * 
 * @param uri : the path snippet to be turned into a rest URL. Leading slash is added if necessary.
 */
GH.Ajax.buildRestUrl = function(uri) {
    if (uri.indexOf('/') != 0) {
        uri = '/' + uri;
    }

	return GH.Ajax.CONTEXT_PATH + GH.Ajax.REST_URL_BASE + uri;
};

/**
 * Extend the result coming from the server with some convenience methods. Basically, we want to
 * avoid checking all over the place if something is there or not and rather have some convenience
 * methods that are more readable.
 */
GH.Ajax.extendResult = function(result) {
    return AJS.$.extend(result, {

        getGlobalErrors : function() {
            return result.error ? result.error.errors : null;
        },
        
        getContextualErrors : function() {
            return result.error ? result.error.contextualErrors : null;
        },
        
        hasGlobalErrors : function() {
            return result.error && result.error.errors;
        },
        
        hasContextualErrors : function() {
            return result.error && result.error.contextualErrors;
        },
        
        addGlobalError : function(error) {
            if (!result.error) result.error = {};
            if (!result.error.errors) result.error.errors = [];
            result.error.errors[result.error.errors.length] = error;
        },
        
        getSuccessObject : function() {
            return result.success;
        },
        
        isSuccess : function() {
            return result.success && ! result.error;
        }
    });
};

/**
 * generic error handler for non 200 replies
 * @param jqXHR
 * @param textStatus
 * @param errorThrown
 */
GH.Ajax._generateErrorHandler = function(onGlobalError) {
    return function(jqXHR, textStatus, errorThrown) {
        // handle case where we aborted the request ourselved
        if (jqXHR.abortMarker) {
            return;
        }

        var result = GH.Ajax.extendResult({});
        //if we've got a timeout we want something better than "abort"
        if(jqXHR.status == 0 && errorThrown == "abort") {
            result.addGlobalError({message: AJS.I18n.getText("gh.ajax.error.timeout")});
        } else {
            result.addGlobalError({message: errorThrown});
        }
        GH.Ajax.handleGlobalErrors(result, onGlobalError);
    }
};