/**
 * Client side validation and error display functions
 */
GH.Validation = {};

/**
 * Validate that an input doesn't have an empty value. If the validation is failing, an error message
 * is displayed next to the field.
 * 
 * @param field : input of type text or textarea
 * @param messageKey : the i18nKey of the message to be displayed
 * @param errorContextElement : (optional) an alternative element where the error should be shown (can be an element or a CSS selector)
 *
 * @returns true if the validation passed, false if an error was shown
 */
GH.Validation.notBlank = function(field, messageKey, errorContextElement) {
    var passed = GH.Validation.applyRules(field, messageKey, errorContextElement, function() {
        
        return AJS.$.trim(AJS.$(field).val()).length > 0;
    });
    
    return passed;
};

/**
 * Validate that an input is a positive integer (empty or >= 0). If the validation is failing, an error message
 * is displayed next to the field.
 *
 * @param field : input of type text or textarea
 * @param messageKey : the i18nKey of the message to be displayed
 * @param errorContextElement : (optional) an alternative element where the error should be shown (can be an element or a CSS selector)
 *
 * @returns true if the validation passed, false if an error was shown
 */
GH.Validation.isPositiveInteger = function(field, messageKey, errorContextElement) {
    var passed = GH.Validation.applyRules(field, messageKey, errorContextElement, function() {
        
        return /^\d*$/.test(AJS.$.trim(AJS.$(field).val()));
    });
    
    return passed;
};

/**
 * If the given rule evaluates to false, the error message will be shown and validation fails.
 * 
 * @param rule : a function that returns a boolean result
 */
GH.Validation.applyRules = function(field, messageKey, errorContextElement, rule) {
    field = AJS.$(field);
    errorCtx = errorContextElement ? AJS.$(errorContextElement) : field;
    
    var passed = rule.call(this);
    if (!passed) {
        GH.Validation.showContextualErrors(errorCtx, messageKey);
        return false;
    }
    
    return true;
};

GH.Validation.showContextualErrors = function(errorCtx, messageKey, param1, param2, param3) {
    // remove previous errors
    GH.Validation.clearContextualErrors(errorCtx);

    // show error message
    errorCtx.after(GH.tpl.rapid.notification.renderContextualErrors({
        errors : [ { message : AJS.I18n.getText(messageKey, param1, param2, param3) } ]
    }));
};

/**
 * Removes all field error messages under or next to the given element
 * 
 * @param element : an element or a CSS selector
 */
GH.Validation.clearContextualErrors = function(element) {
    element = AJS.$(element);
    
    // remove all children that are error messages
    element.find(".ghx-error").remove();
    
    // remove all siblings, in case someone passes in the field directly
    element.siblings(".ghx-error").remove();
};
