/**
 * Inline editable element support.
 * 
 * Binds to a specific element (e.g. given by selector) and turns the element into an edit view when being clicked.
 */
GH.RapidBoard.Util.InlineEditable = {};

//
// Generic helper functions 
//

/**
 * Fetches data-fieldname and data-fieldvalue element attributes.
 */
GH.RapidBoard.Util.InlineEditable.defaultFieldData = function(viewElement, options) {
    var fieldName = viewElement.attr('data-fieldname');
    var fieldValue = viewElement.attr('data-fieldvalue');
    return {
    	fieldName: fieldName,
    	fieldValue: fieldValue
    };
};

/**
 * Default activate function. Focuses the root element
 */
GH.RapidBoard.Util.InlineEditable.defaultActivate = function(editData) {
	editData.editElement.focus();
	editData.editElement.select();
},

/**
 * Renders the inline edit for a textfield, optionally using the provided render function
 * @renderFunction function that renders the markup, typically a soy template. The parameter passed to renderFunction is editData
 */
GH.RapidBoard.Util.InlineEditable.renderTextFieldInlineEdit = function(editData, renderFunction) {
	if (! renderFunction) {
		renderFunction = GH.tpl.rapid.view.renderBasicTextFieldEdit;
	}
	
	// render the element
	var editView = AJS.$(renderFunction(editData));
	
    // currently it's the input directly. Change to .find('input...') if necessary.
    editView.val(editData.fieldValue);
    
    // save on blur
    editView.blur(function(event) {
    	GH.RapidBoard.Util.InlineEditable.submitChanges(editData);
    });

    // handle escape and return
    editView.keydown(function(event) {
        if (event.keyCode == 27) { // escape key
            event.stopPropagation();
            GH.RapidBoard.Util.InlineEditable.cancelEdit(editData);
        }
        else if (event.keyCode == 13) { // return
            event.preventDefault();
            GH.RapidBoard.Util.InlineEditable.submitChanges(editData);
        }
    });
    
    return editView;
};

/**
 * Validates that the text field is not empty, and assigns the new value in case of success.
 */
GH.RapidBoard.Util.InlineEditable.defaultTextValidation = function(editData) {
	// new value
    var newValue = editData.editElement.val();
    editData.newValue = newValue;
    return true;
};


//
// Inline editable fields support
//

/**
 * Register a new inline edit with all the necessary options 
 */
GH.RapidBoard.Util.InlineEditable.register = function(selector, options) {
    
    // defaults
    var effectiveOptions = {
        getData:    GH.RapidBoard.Util.InlineEditable.defaultFieldData,
        preEdit:    function(editData) {},
        renderEdit: GH.RapidBoard.Util.InlineEditable.renderTextFieldInlineEdit,
        activate:   GH.RapidBoard.Util.InlineEditable.defaultActivate,
        validate:   GH.RapidBoard.Util.InlineEditable.defaultTextValidation,
        save:       function(editData) {
                        // directly update the view
                        GH.RapidBoard.Util.InlineEditable.updateView(editData);
                    },
        renderView: GH.RapidBoard.Config.Columns.EditField.renderColumnNameView,
        postEdit:   function(editData) {}
    };
    
    // override the defaults with the given options
    if (options) {
        AJS.$.extend(true, effectiveOptions, options);
    }
    
	AJS.$(selector).live('click', effectiveOptions, GH.RapidBoard.Util.InlineEditable.startEdit);
};

/**
 * Called when a user clicks on the view element.
 */
GH.RapidBoard.Util.InlineEditable.startEdit = function(event) {
	// fetch the options for this edit 
	var options = event.data;
	
	// fetch the edit data from the clicked view element
	var viewElement = AJS.$(this);
	if (options.getData) {
		var editData = options.getData(viewElement, options);
	}
	if(! editData) return;
	editData.viewElement = viewElement;
	editData.options = options;
	
	// invoke pre edit
	if (options.preEdit) {
		options.preEdit(editData);
	}
	
	// render the edit dom
	var editElement = options.renderEdit(editData);
	editData.editElement = editElement;
	
	// replace the view for the edit
	AJS.$(editData.viewElement).replaceWith(editData.editElement);
	
	// finally activate the edit
	if (options.activate) {
		options.activate(editData);
	}
};

/**
 * Cancels a currently edited inline edit.
 */
GH.RapidBoard.Util.InlineEditable.cancelEdit = function(editData) {
	editData.canceled = true;
	
	// replace the edit for the view
	editData.editElement.replaceWith(editData.viewElement);
	
    // remove all error messages, if any
    GH.Validation.clearContextualErrors(editData.viewElement);
    
    // call the post edit callback
    if (editData.options.postEdit) {
    	editData.options.postEdit(editData);
    }
};

/**
 * Triggers submission of edit changes.
 * Validation occurs before submission
 */
GH.RapidBoard.Util.InlineEditable.submitChanges = function(editData) {
	// first validate the changes
	var success = editData.options.validate(editData);
	if (! success) {
		editData.options.activate(editData);
		return;
	}
	
	// save the data
	editData.options.save(editData);
};

/**
 * Updates the view, thus replaces the edit dom by a newly generated view dom.
 */
GH.RapidBoard.Util.InlineEditable.updateView = function(editData) {
	editData.updated = true;
	
	// render the view and replace the edit
	var viewElement = editData.options.renderView(editData);
    AJS.$(editData.editElement).replaceWith(viewElement);
    editData.viewElement = viewElement;
    
    // remove all error messages, if any
    GH.Validation.clearContextualErrors(editData.viewElement);
    
    // we are done, call the post function
	if (editData.options.postEdit) {
		editData.options.postEdit(editData);
	}
};
