/**
 * Swimlanes configuration tab
 */
GH.RapidBoard.Config.Swimlanes = {};

/**
 * Holds the model for the configuration.
 */
GH.RapidBoard.Config.Swimlanes.model = undefined;

/**
 * Initializes the swimlanes config.
 */
GH.RapidBoard.Config.Swimlanes.init = function(data) {
	GH.RapidBoard.Config.Swimlanes.model = data.swimlanesConfig.success;
	
    GH.RapidBoard.Config.Swimlanes.initializeTable();
};

/**
 * Renders the edit form using the locally stored model
 */
GH.RapidBoard.Config.Swimlanes.initializeTable = function() {
    var model = GH.RapidBoard.Config.Swimlanes.model;
    var canEdit = model.canEdit;
    var rapidViewId = model.rapidViewId 
    var swimlanes = model.swimlanes;
    
    // apply the fact that we are readonly to all swimlanes, as the row renderer only gets the data object for the row
    // only do this if we can't edit as this won't cut it for updated rows (there the response of the rest calls is directly
    // used, so we have no way of adapting the entries upon success.
    // Not setting this for canEdit = true ensures nobody misuses readonly for that matter...
    if (! canEdit) {
    	for (var i=0; i < swimlanes.length; i++) {
    		swimlanes[i].readonly = true;
    	}
    }
    
    AJS.$('#ghx-config-swimlanes').html(GH.tpl.rapid.view.renderSwimlaneConfigView({canEdit:canEdit}));
    
    // setup the restful table
    var restfulTable = new JIRA.RestfulTable({
        el: jQuery("#ghx-swimlane-table"), // table to add entries to. Entries are appended to the tables <tbody> element
        url: GH.Ajax.buildRestUrl('/swimlanes/'+rapidViewId+'/'), //rest resource for collection
        editable: canEdit,
        reorderable: canEdit,
        entries: swimlanes,
        views: {
            editRow: GH.RapidBoard.Config.Swimlanes.EditRow,
            row: GH.RapidBoard.Config.Swimlanes.Row
        }
    });

    // Only make rows with js-sortable class sortable 
    restfulTable.$tbody.sortable('option', 'items', '> .js-sortable');

    // register jql auto complete
    if (model.canEdit) {
    	GH.JQLAutoComplete.initialize('ghx-swimlane-jql-create', 'ghx-swimlane-jql-errormsg-create');
    }
};

/**
 * Table view row object 
 */
GH.RapidBoard.Config.Swimlanes.Row = JIRA.RestfulTable.Row.extend({

    initialize: function () {
        // call super
        JIRA.RestfulTable.Row.prototype.initialize.apply(this, arguments);

        // crap work around to handle backbone not extending events
        // (https://github.com/documentcloud/backbone/issues/244)
        this.events["click .js-swimlane-delete"] = "_delete";
        this.delegateEvents();
    },

    _delete: function (e) {
        this._openDialog(GH.RapidBoard.Config.Swimlanes.DeleteForm,
                "ghx-swimlane-" + this.model.get("id") + "-delete-dialog");

        e.preventDefault();
    },

    render: function () {
    	// set the correct class onto the row - either the row is sortable or it is the default row
    	if (! this.model.get('isDefault')) {
    		this.$el.addClass('js-sortable');
    	} else {
    		this.$el.addClass('ghx-default-swimlane');
    	}
        this.$el.html(GH.tpl.rapid.view.renderSwimlaneRow(this.model.toJSON()));
        return this;
    }
});

/**
 * Table edit row object
 */
GH.RapidBoard.Config.Swimlanes.EditRow = JIRA.RestfulTable.EditRow.extend({

    render: function (renderData) {
        this.$el.html(GH.tpl.rapid.view.renderSwimlaneEditRow(renderData));
        
        return this;
    },

    // used to handle JQL errors - so form is not submitted
    submit: function () {
        var values = this.$el.serializeToObject();
        var oldJql = this.model.attributes.query;
        var newJql = values && values.query ? values.query : "";
        if(oldJql)
        {
            //If we have an old query, then we have an edit event
            AJS.$(AJS).trigger("gh.swimlane.edit", {
                rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
                oldJql: oldJql,
                newJql: newJql
            });
        } else {
            // else we have a create event
            AJS.$(AJS).trigger("gh.swimlane.create", {
                rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
                jql: newJql
            });
        }

        return JIRA.RestfulTable.EditRow.prototype.submit.apply(this, arguments);
    }
});

/**
 * Swimlane delete form
 */
GH.RapidBoard.Config.Swimlanes.DeleteForm = Backbone.View.extend({

    /**
     * Destroys model on server
     *
     * @param {Object} values
     * @param complete
     * @return {JIRA.Admin.Version.DeleteForm}
     */
    submit: function (values, row, complete) {

        this.$(".throbber").addClass("loading");

        AJS.$(AJS).trigger("gh.swimlane.delete", {
            rapidViewId: GH.RapidBoard.Config.editedRapidViewId,
            jql: row.model.attributes.query
        });

        this.model.destroy({
            data: values,
            success: function () {
                complete();
            },
            error: function () {
                complete();
            }
        });

        return this;
    },

    /**
     * Renders delete form. This differs from standard render methods, as it requires async request/s to the server.
     * As a result when this method is calle the first argument is a function that is called when the content has been
     * rendered.
     *
     * @param {function} ready - callback to declare content is ready
     * @return {JIRA.Admin.Version.DeleteForm}
     */
    render: function (ready) {

        this.el.innerHTML = GH.tpl.rapid.view.deleteForm({
            message: AJS.I18n.getText('gh.rapid.swimlane.delete.confirm', soy.$$escapeHtml(this.model.get('name')))
        });
        AJS.$(this.el).addClass('ghx-dialog');
        ready.call(this, this.el);

        return this;
    }
});
