/**
 * Central state manager for the Rapid Board.
 */
GH.RapidBoard.State = {};

/**
 * Holds all local data. Until a page reload, all state is solely read from this state - not from the local or session storage.
 */
GH.RapidBoard.State.data = {};


/** Set a data value. */
GH.RapidBoard.State.setDataValue = function(name, value) {
	if (_.isUndefined(value) || _.isNull(value)) {
		delete GH.RapidBoard.State.data[name];
	} else {
        GH.RapidBoard.State.data[name] = value;
	}
};

/** Get a data value. */
GH.RapidBoard.State.getDataValue = function(name) {
	return GH.RapidBoard.State.data[name];
};


//
// Individual state variables
//

/**
 * Get the current view name
 */
GH.RapidBoard.State.getView = function() {
	return GH.RapidBoard.State.getDataValue('view') || '';
};
GH.RapidBoard.State.getStoredView = function() {
	return GH.storage.get('gh.view') || '';
};

/**
 * Is the details view currently opened
 */
GH.RapidBoard.State.isDetailsViewOpened = function() {
	return GH.RapidBoard.State.getView() == 'detail';
};

/**
 * Sets whether the chart view is opened or not
 */
GH.RapidBoard.State.isChartViewOpened = function(opened) {
	return GH.RapidBoard.State.getView() == 'reporting';
};

/**
 * Sets the name of the current view.
 * Current known views are '', 'detail', 'chart'
 */
GH.RapidBoard.State.setView = function(view) {
	// store the current view (to be able to go back in case of a clear)
	var currentView = GH.RapidBoard.State.getView();
	GH.RapidBoard.State.setDataValue('retainedView', currentView);
	
	GH.RapidBoard.State.setDataValue('view', view);
	GH.storage.put('gh.view', view);
};

/**
 * Sets whether the chart view is opened or not
 */
GH.RapidBoard.State.setChartViewOpened = function(opened) {
	// special case for closing the chart view. if we are in the detail view, return to it
	if (opened) {
		GH.RapidBoard.State.setView('reporting');
	} else {
		var retainedView = GH.RapidBoard.State.getDataValue('retainedView') == 'detail' ? 'detail' : '';
		GH.RapidBoard.State.setView(retainedView);
	}
};

/**
 * Sets whether the intro chart is open
 */
GH.RapidBoard.State.setChartIntro = function(introActive) {
    if(!!introActive) {
        GH.RapidBoard.State.setDataValue('intro', true);
    } else {
        GH.RapidBoard.State.setDataValue('intro', undefined);
    }
};

/**
 * Returns whether the intro chart is open
 */
GH.RapidBoard.State.getChartIntro = function() {
    return GH.RapidBoard.State.getDataValue('intro') || false;
};

/**
 * Sets the details view state
 */
GH.RapidBoard.State.setDetailsViewOpened = function(opened) {
	GH.RapidBoard.State.setView(opened ? 'detail' : '');
};

/**
 * Get the current rapid view id
 */
GH.RapidBoard.State.getRapidViewId = function() {
	return GH.RapidBoard.State.getDataValue('rapidViewId');
};

/**
 * Get the last stored rapid view id
 */
GH.RapidBoard.State.getMostRecentRapidViewId = function() {
	return GH.storage.get('gh.latestRapidViewId');
};

/**
 * Set the current rapid view id
 */
GH.RapidBoard.State.setRapidViewId = function(rapidViewId) {
	var data = GH.RapidBoard.State.data;
	
	// set the new value
	GH.RapidBoard.State.setDataValue('rapidViewId', rapidViewId);
	
	// store the id in the local storage
	GH.storage.put('gh.latestRapidViewId', rapidViewId);
};

/**
 * Get the selected issue key
 */
GH.RapidBoard.State.getSelectedIssueKey = function() {
    return GH.RapidBoard.State.getDataValue('selectedIssueKey') || null;
};
GH.RapidBoard.State.getStoredSelectedIssueKey = function() {
	return GH.storage.get('gh.selectedIssueKey') || null;
};

/**
 * Set the selected issue key
 */
GH.RapidBoard.State.setSelectedIssueKey = function(issueKey) {
	GH.RapidBoard.State.setDataValue('selectedIssueKey', issueKey);
	GH.storage.put('gh.selectedIssueKey', issueKey);
};

/**
 * Sets the chart type state.
 * Chart type setting is stored per-RapidView.
 */
GH.RapidBoard.State.setChartType = function(chartType) {
    var settings = GH.RapidBoard.State.getBoardSettings();
    if (_.isUndefined(chartType) || _.isNull(chartType)) {
        delete settings.chartType;
    } else {
        settings.chartType = chartType;
    }
    GH.RapidBoard.State.setBoardSettings(settings);
};

/**
 * Get the chart type state for the current RapidView
 */
GH.RapidBoard.State.getChartType = function() {
	return GH.RapidBoard.State.getBoardSettings()["chartType"] || 'controlChart';
};

/**
 * Sets the current charting time frame.
 * Time frame setting is stored per-RapidView.
 */
GH.RapidBoard.State.setTimeFrame = function(timeFrame) {
    var settings = GH.RapidBoard.State.getBoardSettings();
    if (_.isUndefined(timeFrame) || _.isNull(timeFrame)) {
        delete settings.timeFrame;
    } else {
        settings.timeFrame = timeFrame;
    }
    GH.RapidBoard.State.setBoardSettings(settings);
};

/**
 * Get the charting time frame for the current RapidView
 */
GH.RapidBoard.State.getTimeFrame = function() {
    var val = GH.RapidBoard.State.getBoardSettings()["timeFrame"];
    if(_.isUndefined(val) || _.isNull(val)) {
        return 7;
    } else {
        return val;
    }
};


// RapidBoard id specific settings.

/**
 * Sets the board settings. This object contains all the board specific settings. The implementation
 * ensures that it always returns the right settings for the given rapidBoardId.
 */
GH.RapidBoard.State.getBoardSettings = function(rapidViewId) {
	if (_.isUndefined(rapidViewId) || _.isNull(rapidViewId)) rapidViewId = GH.RapidBoard.State.getRapidViewId(); 
	var id = rapidViewId || "00000";
	var settings = GH.RapidBoard.State.getDataValue('boardSettings.' + id) || GH.storage.get('gh.boardSettings.'+id);
	if (! settings) {
		// set an empty settings object
		settings = {};
		GH.RapidBoard.State.setDataValue('boardSettings.' + id, settings);
	}
	
	return settings || {};
};

/** Set the board settings object for the current rapid board. */
GH.RapidBoard.State.setBoardSettings = function(settings) {
	var id = GH.RapidBoard.State.getRapidViewId();
	if (! id) {
		// don't bother storing these settings
		return;
	}
	
	// stored it locally and in the storage
	GH.RapidBoard.State.setDataValue('boardSettings.'+id, settings);
	GH.storage.put('gh.boardSettings.' + id, settings);
};


// Collapsed swimlanes

GH.RapidBoard.State.isSwimlaneCollapsed = function(swimlaneId) {
	var lanes = GH.RapidBoard.State.getBoardSettings()["collapsedSwimlanes"] || [];
	return lanes.indexOf(swimlaneId) > -1;
};

GH.RapidBoard.State.setSwimlaneCollapsed = function(swimlaneId, collapsed) {
	var settings = GH.RapidBoard.State.getBoardSettings();
	var lanes = settings.collapsedSwimlanes || [];
	if (collapsed) {
		if (lanes.indexOf(swimlaneId) < 0) {
			lanes.push(swimlaneId);
			settings.collapsedSwimlanes = lanes;
		}
	} else {
		// remove the index, cleanup array if empty
		var index = lanes.indexOf(swimlaneId);
		if (index < 0) return;
		lanes.splice(index, 1);
		if (lanes.length > 0) {
			settings.collapsedSwimlanes = lanes;
		} else {
			delete settings.collapsedSwimlanes
		}
	}
	GH.RapidBoard.State.setBoardSettings(settings);
};

// active quick filters

/**
 * Get the active quick filters
 */
GH.RapidBoard.State.getActiveQuickFilters = function (rapidViewId) {
	return GH.RapidBoard.State.getBoardSettings(rapidViewId)["activeQuickFilters"] || [];
};

/**
 * Set the active quick filters
 */
GH.RapidBoard.State.setActiveQuickFilters = function (activeQuickFilters) {
	var settings = GH.RapidBoard.State.getBoardSettings();
	if (activeQuickFilters.length > 0) {
		settings.activeQuickFilters = activeQuickFilters;
	} else {
		delete settings.activeQuickFilters;
	}
	GH.RapidBoard.State.setBoardSettings(settings);
};



/**
 * Get the generic board setting
 */
GH.RapidBoard.State.getBoardSetting = function (rapidViewId, propertyName, defaultValue) {
	var value = GH.RapidBoard.State.getBoardSettings(rapidViewId)[propertyName]
	if (value !== undefined) return value;
	return defaultValue; 
};

/**
 * Set a generic board setting
 */
GH.RapidBoard.State.setBoardSetting = function (propertyName, value) {
	var settings = GH.RapidBoard.State.getBoardSettings();
	if (value !== undefined) {
		settings[propertyName] = value;
	} else {
		delete settings[propertyName];
	}
	GH.RapidBoard.State.setBoardSettings(settings);
};

/**
 * Get a number array board setting
 */
GH.RapidBoard.State.getNumberArrayBoardSetting = function (rapidViewId, propertyName, defaultValue) {
	var value = GH.RapidBoard.State.getBoardSettings(rapidViewId)[propertyName] || defaultValue;
	if (! _.isArray(value)) {
		AJS.log("Property is not of type array, returning an empty array: " + propertyName);
		return [];
	}
	// TODO: handle non-number case better
	_.each(value, function(entry) {
		if (!_.isNumber(entry)) {
			AJS.log("Warning: Property value is not of type number: " + entry);
		}
	});
	return value;
};

/**
 * Set a number array setting
 */
GH.RapidBoard.State.setNumberArrayBoardSetting = function (propertyName, value) {
	var settings = GH.RapidBoard.State.getBoardSettings();
	if (value !== undefined) {
		// ensure it is indeed a number array
		if (! _.isArray(value)) {
			AJS.log("Property is not of type array, returning an empty array: " + propertyName);
			return;
		}

		// TODO: handle non-number case better
		_.each(value, function(entry) {
			if (!_.isNumber(entry)) {
				AJS.log("Warning: Property value is not of type number: " + entry);
			}
		});
		
		settings[propertyName] = value;
	} else {
		delete settings[propertyName];
	}
	GH.RapidBoard.State.setBoardSettings(settings);
};


// state pushing / replacement

/**
 * Pushes the current state onto the url.
 */
GH.RapidBoard.State.pushState = function() {
	GH.RapidBoard.UrlState.pushState();
};

/**
 * Replaces the current state onto the url.
 */
GH.RapidBoard.State.replaceState = function() {
	GH.RapidBoard.UrlState.replaceState();
};

/**
 * Initializes the state of the rapid board.
 * This function is called exactly once, on page load
 */
GH.RapidBoard.State.initalizeState = function() {
	// load defaults from local/session storage
	// intentionally do NOT load the rapidViewId, as this is handled differently (see RabidBoard.loadBoardData)
	GH.RapidBoard.State.setSelectedIssueKey(GH.RapidBoard.State.getStoredSelectedIssueKey());
	GH.RapidBoard.State.setView(GH.RapidBoard.State.getStoredView());
	
	// now load the state from available url parameters
	if (GH.RapidBoard.UrlState.hasUrlState()) {
		GH.RapidBoard.UrlState.loadState();
	}
	
	// replace the state as currently set
	GH.RapidBoard.UrlState.replaceState();
	
    // now as a last step, initialize the UrlState, which will listen to state changes
    GH.RapidBoard.UrlState.init();
};
