/**
 * Manages the state of the url.
 */
GH.RapidBoard.UrlState = {};

GH.RapidBoard.UrlState.dateFormat = "%Y-%m-%d";

/**
 * 
 * Url parameter normalization utils
 * 
 */

/**
 * Normalizes a string param into a number.
 * @return the number or null if not a valid number, NaN and Infinite.
 */
GH.RapidBoard.UrlState.normalizeNumberParam = function(value, defaultValue) {
    if (_.isUndefined(defaultValue)) defaultValue = null;
	if (value === undefined) return defaultValue;
	if (value === null) return defaultValue;
	var numberVal = parseInt(value, 10); // 10 radix
	if (!isFinite(numberVal)) return defaultValue; // _.isNumber returns true for Infinite, isFinite returns true for null. both return false for undefined.
	return numberVal;
};

/**
 * Normalizes a boolean param
 */
GH.RapidBoard.UrlState.normalizeBooleanParam = function(value, defaultValue) {
    if (_.isUndefined(defaultValue)) defaultValue = null;
	if (value === undefined) return defaultValue;
	if (value === null) return defaultValue;
    if (value === "false") return false;
    return new Boolean(value);
};

/**
 * Normalizes a date param of the form yyyy-mm-dd
 * @return the date object or null if not set. Note that an invalid date will translate into "now"
 */
GH.RapidBoard.UrlState.normalizeDateParam = function(value, defaultValue) {
    if (_.isUndefined(defaultValue)) defaultValue = null;
	if (value === undefined) return defaultValue;
	if (value === null) return defaultValue;
	var dateVal = Date.parseDate(value, GH.RapidBoard.UrlState.dateFormat);
	return dateVal;
};

/**
 * Translates a parameter value (which might be undefined, a string or an array of strings) into a number array.
 */
GH.RapidBoard.UrlState.normalizeNumberArrayParam = function(value) {
	var res = [];
	if (_.isString(value)) {
		// convert to number
		var numberVal = GH.RapidBoard.UrlState.normalizeNumberParam(value);
		if (numberVal !== null) {
			res.push(numberVal);
		}
	} else if(_.isArray(value)) {
		// convert each value
		_.each(value, function(v) {
			var numberVal = GH.RapidBoard.UrlState.normalizeNumberParam(v);
			if (numberVal !== null) {
				res.push(numberVal);
			}
		});
	}
	return res;
};

/**
 * Normalizes a string param.
 * @param defaultValue. What value to use in case in case of null, undefined or an empty string. defaultValue defaults to null
 */
GH.RapidBoard.UrlState.normalizeStringParam = function(value, defaultValue) {
	// ensure we have a default value
	if (_.isUndefined(defaultValue)) defaultValue = null;
	
	// for arrays we use the first array element
	if(_.isArray(value) && value.length > 0) {
		var val = value[0]; // use the first value in case we got multiple ones
		if (val !== '') return val;
		else return defaultValue;
	}
	// for string simply apply the normalization
	if (_.isString(value)) {
		if (value !== '') return value;
		else return defaultValue;
	}
	// fall back to defaulValue
	return defaultValue;
};

/**
 * Removes empty arrays, strings, undefined, null parameters from a normalized parameter object
 */
GH.RapidBoard.UrlState.removeEmptyParams = function(normalized) {
	var params = {};
    _.each(normalized, function(value, key) {
    	if (_.isArray(value)) {
    		if (value.length > 0) {
    			params[key] = value;
    		}
    	}
    	else if (_.isString(value)) {
    		// remove empty strings
    		if (!_.isEmpty(value)) {
    			params[key] = value;
    		}
    	}
    	else if (value === null || value === undefined) {} // ignore
    	else {
    		// if in doubt add
    		params[key] = value;
    	}
    });
    return params;
};

/**
 * Get the view state as request parameters.
 * The parameters are concatenated using &, no ? is prepended
 */
GH.RapidBoard.UrlState.toUrlParameterString = function(params) {
    var query = '';
    _.each(params, function(value, key) {
    	if (_.isArray(value)) {
    		_.each(value, function(e) {
    			if (query.length > 0) query += '&';
    			query += key + "=" + encodeURIComponent(value);
    		});
    	}
    	else {
    		if (query.length > 0) query += '&';
    		query += key + "=" + encodeURIComponent(value);
    	}
    });
    return query;
};


/**
 * 
 * Normalized <===> url
 * 
 */

/**
 * Returns the url params in a normalized way.
 */
GH.RapidBoard.UrlState.getNormalizedUrlState = function() {
	var params = GH.RapidBoard.UrlState.getCurrentUrlState();
	var normalized = {};
	
	// view
	normalized['view'] = GH.RapidBoard.UrlState.normalizeStringParam(params['view'], ''); // empty string by default
	
	// rapid view id
	normalized['rapidView'] = GH.RapidBoard.UrlState.normalizeNumberParam(params['rapidView']); // null by default

	// view dependent variables
	switch(normalized['view']) {
	case 'reporting':
		// chart type
		normalized['chart'] = GH.RapidBoard.UrlState.normalizeStringParam(params['chart'], 'controlChart'); // controlChart by default
        
        //whether or not the intro is active
        normalized['intro'] = GH.RapidBoard.UrlState.normalizeBooleanParam(params['intro'], false); // false by default
        if(normalized['intro'] !== false) {
            normalized['intro'] = true
        }
        
        // time frames
        var days = GH.RapidBoard.UrlState.normalizeNumberParam(params['days'], 0);
        var from = GH.RapidBoard.UrlState.normalizeDateParam(params['from'], null);
        var to = GH.RapidBoard.UrlState.normalizeDateParam(params['to'], null);
        if (from !== null && to !== null) {
        	normalized['days'] = -1;
        	normalized['from'] = from.getTime();
        	normalized['to'] = to.getTime();
        } else {
        	normalized['days'] = days;
        	normalized['from'] = 0;
        	normalized['to'] = 0;
        }
        
        // filters
        normalized['swimlane'] = GH.RapidBoard.UrlState.normalizeNumberArrayParam(params['swimlane']); // empty by default
        normalized['quickFilter'] = GH.RapidBoard.UrlState.normalizeNumberArrayParam(params['quickFilter']); // empty by default
        normalized['column'] = GH.RapidBoard.UrlState.normalizeNumberArrayParam(params['column']); // empty by default
        
        break;
		
	case 'detail':
		// intentionally fall through to default, shares same url params
	default:
		// selected issue
		normalized['selectedIssue'] = GH.RapidBoard.UrlState.normalizeStringParam(params['selectedIssue'], null); // null by default
	
		// quick filter
		normalized['quickFilter'] = GH.RapidBoard.UrlState.normalizeNumberArrayParam(params['quickFilter']); // empty by default
	}
	
	return normalized;
};

GH.RapidBoard.UrlState.toUrlState = function(normalized) {

	// remove empty values
	var params = GH.RapidBoard.UrlState.removeEmptyParams(normalized);

    // remove from/to in case we are in relative mode
    if (params['days'] != -1) {
    	delete params['from'];
    	delete params['to'];
    } else {
    	// otherwise transform the dates into yyyy-mm-dd values
    	params['from'] = new Date(params['from']).print(GH.RapidBoard.UrlState.dateFormat);
    	params['to'] = new Date(params['to']).print(GH.RapidBoard.UrlState.dateFormat);
    }
    
    // remove the days parameter if we are in absolute mode (-1) or if its the all time option
    if(params['days'] <= 0) {
        delete params['days'];
    }
    
    // remove intro param if it is false
    if(params['intro'] === false) {
        delete params['intro'];
    }
    // replace true by 1
    if(params['intro']) {
        params['intro'] = 1;
    }

	return params
};

/**
 * Updates the url state with the current values
 */
GH.RapidBoard.UrlState.pushState = function() {
	// fetch then normalized rapid board state
	var normalized = GH.RapidBoard.UrlState.getNormalizedRapidBoardState();
	
	// remove empty values
	var params = GH.RapidBoard.UrlState.toUrlState(normalized);
	
	// push the state
	GH.State.push(params);
};

/**
 * Replaces the url state with the current values
 */
GH.RapidBoard.UrlState.replaceState = function() {
	// fetch then normalized rapid board state
	var normalized = GH.RapidBoard.UrlState.getNormalizedRapidBoardState();
	
	// remove empty values
	var params = GH.RapidBoard.UrlState.toUrlState(normalized);
	
	// replace the state
	GH.State.replace(params);
};


/**
 * 
 * Board <===> normalized state
 * 
 */

/**
 * Returns the rapid board state in a normalized way
 */
GH.RapidBoard.UrlState.getNormalizedRapidBoardState = function() {
	var normalized = {};
	
	// rapid view id
	normalized['rapidView'] = GH.RapidBoard.State.getRapidViewId();
	
	// selected view
	var view = GH.RapidBoard.State.getView();
	normalized['view'] = view;  // by default an empty string
	
	// view dependent stuff
	switch(view) {
	case 'reporting':		
		// chart type
		normalized['chart'] = GH.RapidBoard.State.getChartType();
		
        // are we on chart intro
        normalized['intro'] = GH.RapidBoard.State.getChartIntro();
        
    	// load the days, from, to and filter values from the board settings
        // TODO: default value evaluation should NOT be here
        normalized['days'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.days', 7);
    	if (normalized['days'] == -1) {
    		normalized['from'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.from');
    		normalized['to'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.timeframe.to');
    	} else {
    		normalized['from'] = 0;
    		normalized['to'] = 0;
    	}
    	
    	normalized['column'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.filters.columns.'+normalized['chart']); // columns are chart type specific
    	normalized['swimlane'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.filters.swimlanes');
    	normalized['quickFilter'] = GH.RapidBoard.State.getBoardSetting(null, 'chart.filters.quickFilters');
        
		break;
		
	case 'detail':
		// intentionally fall through
	default:
		// quick filter
		normalized['quickFilter'] = GH.RapidBoard.State.getActiveQuickFilters(); // always fetch quick filters, even none are active
	
		// selected issue
		normalized['selectedIssue'] = GH.RapidBoard.State.getSelectedIssueKey();
	}
	
	return normalized;
};

/**
 * Updates the rapid board state with the url state
 */
GH.RapidBoard.UrlState.loadState = function() {
	// fetch the normalized url params
	var normalized = GH.RapidBoard.UrlState.getNormalizedUrlState();
	
	if (normalized['view'] !== undefined) 			GH.RapidBoard.State.setView(normalized['view']);
	if (normalized['rapidView'] !== undefined) 		GH.RapidBoard.State.setRapidViewId(normalized['rapidView']);
	if (normalized['selectedIssue'] !== undefined) 	GH.RapidBoard.State.setSelectedIssueKey(normalized['selectedIssue']);
	if (normalized['quickFilter'] !== undefined)    GH.RapidBoard.State.setActiveQuickFilters(normalized['quickFilter']);
	
	if (normalized['chart'] !== undefined) 			GH.RapidBoard.State.setChartType(normalized['chart']);
    if (normalized['intro'] !== undefined)          GH.RapidBoard.State.setChartIntro(normalized['intro']);
    if (normalized['days'] !== undefined)           GH.RapidBoard.State.setBoardSetting('chart.timeframe.days', normalized['days']);
    if (normalized['from'] !== undefined)           GH.RapidBoard.State.setBoardSetting('chart.timeframe.from', normalized['from']);
    if (normalized['to'] !== undefined)             GH.RapidBoard.State.setBoardSetting('chart.timeframe.to', normalized['to']);
    
    if (normalized['swimlane'] !== undefined)      GH.RapidBoard.State.setBoardSetting('chart.filters.swimlanes', normalized['swimlane']);
    if (normalized['quickFilter'] !== undefined)    GH.RapidBoard.State.setBoardSetting('chart.filters.quickFilters', normalized['quickFilter']);
    if (normalized['column'] !== undefined)        GH.RapidBoard.State.setBoardSetting('chart.filters.columns.'+normalized['chart'], normalized['column']);
};


/**
 * 
 * Current state fetching, comparison board <===> url
 * 
 */

/**
 * Get the current url state, not normalized
 */
GH.RapidBoard.UrlState.getCurrentUrlState = function() {
	return GH.State.toViewState(window.location.href);
};

/**
 * Is there any state currently encoded in the url. This merely checks whether there are parameters present
 */
GH.RapidBoard.UrlState.hasUrlState = function() {
	return !_.isEmpty(GH.RapidBoard.UrlState.getCurrentUrlState());
};

/**
 * Compares the state of the url to the current internal state
 */
GH.RapidBoard.UrlState.isUrlStateUpToDate = function() {
	// fetch internal and url state
	var boardNormalized = GH.RapidBoard.UrlState.getNormalizedRapidBoardState();
	var urlNormalized = GH.RapidBoard.UrlState.getNormalizedUrlState();
	
	// compare
	return _.isEqual(boardNormalized, urlNormalized);
};

/**
 * Get the view state as request parameters.
 * The parameters are concatenated using &, no ? is prepended
 */
GH.RapidBoard.UrlState.getViewStateRequestParameterString = function() {
	// fetch then normalized rapid board state
	var normalized = GH.RapidBoard.UrlState.getNormalizedRapidBoardState();
	
	// remove empty values
	var params = GH.RapidBoard.UrlState.removeEmptyParams(normalized);
	
	// transform to url parameter string
	return GH.RapidBoard.UrlState.toUrlParameterString(params);
};


/**
 * Called when the history changed.
 */
GH.RapidBoard.UrlState.historyChanged = function() {
	if (GH.RapidBoard.UrlState.isUrlStateUpToDate()) return;

	// apply the new state
	//alert('history changed');
	GH.RapidBoard.UrlState.loadState();
	
	// history really changed, inform RapidBoardState
	// TODO:
    // for now we simply reload the complete model
    //GH.RapidBoard.UrlState.state = data;
    GH.RapidBoard.reload();
};

/**
 * Initializes the view state object
 */
GH.RapidBoard.UrlState.init = function() {
    // register a listener for history change events
	// Delay registration, pre html 5 browsers fake the events, which is fired after the board is initialized
	// (we specifically want to ignore the initial pushState event)
	setTimeout(function() {
		GH.State.registerStateChangeListener(GH.RapidBoard.UrlState.historyChanged);
	}, 200);
};
