/**
 * Detail fields component.
 *
 * Manages the view/edit state of fields as well as general field rendering
 */
GH.RapidBoard.DetailFields = {};

//
// Edit info support
//

/**
 * Holds the editInfo objects for all fields currently being edited.
 */
GH.RapidBoard.DetailFields.editInfos = [];

/**
 * Get the number of currently edited fields
 */
GH.RapidBoard.DetailFields.getEditingCount = function() {
	return GH.RapidBoard.DetailFields.editInfos.length; 
};

/**
 * Is the field edited?
 */
GH.RapidBoard.DetailFields.isEditing = function(fieldId) {
	return GH.RapidBoard.DetailFields.getEditInfo(fieldId) != false; 
};

/**
 * Get the info object for a currently edited field
 */
GH.RapidBoard.DetailFields.getEditInfo = function(fieldId) {
	for (var i=0; i < GH.RapidBoard.DetailFields.editInfos.length; i++) {
		if (GH.RapidBoard.DetailFields.editInfos[i].fieldData.id == fieldId) {
			return GH.RapidBoard.DetailFields.editInfos[i];
		}
	}
	return false;
};

/**
 * Add an editInfo object
 */
GH.RapidBoard.DetailFields.addEditInfo = function(editInfo) {
	GH.RapidBoard.DetailFields.editInfos.push(editInfo);
};

/**
 * Remove an editInfo object
 */
GH.RapidBoard.DetailFields.removeEditInfo = function(editInfo) {
	GH.RapidBoard.DetailFields.editInfos = AJS.$.grep(GH.RapidBoard.DetailFields.editInfos,
		function(value) {
	    	return value.fieldData.id != editInfo.fieldData.id;
		}
	);
};

/**
 * Clears the edit data
 */
GH.RapidBoard.DetailFields.clearEditData = function() {
	GH.RapidBoard.DetailFields.editInfos = [];	
};

//
// Field rendering
//

/**
 * Renders the fields on the details view.
 * @param viewData the view data of the current issue
 * @param issueFields the div container for all fields
 */
GH.RapidBoard.DetailFields.renderFields = function(viewData, issueFields) {
    // allocate fields to module groups
    // note: this is hardcoded now because in the future this might change
    var peopleFields = [], detailsFields = [], datesFields = [], timeTrackingFields = [], descriptionFields = [], issueKeyField = null, summaryField = null;
    for (var i=0; i < viewData.fields.length; i++) {
        var field = viewData.fields[i];
        if (field.id == 'created' || field.id == 'updated') {
            datesFields.push(field);
        } else if (field.id == 'reporter' || field.id == 'assignee') {
            peopleFields.push(field);
        } else if (field.id == 'issuekey') {
            issueKeyField = field;
        } else if (field.id == 'description' || field.id == 'summary') {
            descriptionFields.push(field);
        } else if (field.id == 'timetracking_remainingestimate') {
            timeTrackingFields.push(field);
        } else {
            // everything else is in 'details'
            detailsFields.push(field);
        }
    }

    // first render the issue key
    var issueKeyDom = GH.RapidBoard.DetailFields.renderField(viewData, issueKeyField);
    issueFields.append(issueKeyDom);

    // next render the description module
    var descriptionModuleDom = GH.RapidBoard.DetailFields.renderModule(viewData, descriptionFields, 'ghx-description-module');
    issueFields.append(descriptionModuleDom);

    // next render the details module
    var detailsModuleDom = GH.RapidBoard.DetailFields.renderModule(viewData, detailsFields, 'ghx-details-module');
    issueFields.append(detailsModuleDom);

    // next render the people module
    var peopleModuleDom = GH.RapidBoard.DetailFields.renderModule(viewData, peopleFields, 'ghx-people-module');
    issueFields.append(peopleModuleDom);

    // next render the dates module
    var datesModuleDom = GH.RapidBoard.DetailFields.renderModule(viewData, datesFields, 'ghx-dates-module');
    issueFields.append(datesModuleDom);

    // next render the time tracking module
    var timeTrackingModuleDom = GH.RapidBoard.DetailFields.renderModule(viewData, timeTrackingFields, 'ghx-timetracking-module');
    issueFields.append(timeTrackingModuleDom);

    // clear out the edit infos
    GH.RapidBoard.DetailFields.editInfos = [];
};

/**
 * Renders an entire 'module' (grouping) of fields
 * @param viewData the view data of the current issue
 * @param fields a list of fields which will be rendered inside this module
 * @param moduleId the id to ascribe to this module
 * @return the DOM element of the entire module structure (to be appended back into the issueFields)
 */
GH.RapidBoard.DetailFields.renderModule = function(viewData, fields, moduleId) {
    // module title is set on each field -- just grab it from the first field
    var moduleTitle = fields[0].group;

    // render the module structure
    var moduleStructure = AJS.$(GH.tpl.board.x.renderModuleStructure({
        'moduleId' : moduleId,
        'moduleTitle' : moduleTitle
    }));

    // get a reference to the container for fields within this module
    var moduleFieldsDom = moduleStructure.find('dl.mod-content');
    for (var i=0; i < fields.length; i++) {
        // render each field and append it to the above container
        var fieldDom = GH.RapidBoard.DetailFields.renderField(viewData, fields[i]);
        moduleFieldsDom.append(fieldDom);
    }

    // return the whole structure
    return moduleStructure;
};

/**
 * Renders a single field
 */
GH.RapidBoard.DetailFields.renderField = function(viewData, field) {
	var fieldDom = AJS.$(GH.tpl.board.x.renderFieldStructure({issueDetails: viewData, field: field}));
	var fieldContent = GH.RapidBoard.DetailFields.renderFieldContent(field);
	fieldDom.filter('dd').html(fieldContent);
	return fieldDom;
};

/**
 * Update the view for an updated field
 */
GH.RapidBoard.DetailFields.updateField = function(field) {
	// fetch the editInfo object for this field
	var editInfo = GH.RapidBoard.DetailFields.getEditInfo(field.id);
	if (! editInfo) return;
	
	// we only update the content of the field
	var fieldContent = GH.RapidBoard.DetailFields.renderFieldContent(field);
	editInfo.viewDd.html(fieldContent);
	
	// finally hide the edit and show the view
	GH.RapidBoard.DetailFields.hideEdit(editInfo);
};

/**
 * Ask the renderer to provide the markup for the given field
 */
GH.RapidBoard.DetailFields.renderFieldContent = function(field) {
	var fieldContent;
	switch (field.renderer) {
        case 'html':
            fieldContent = GH.tpl.board.x.renderHtmlView({field: field});
            break;
        case 'date':
            fieldContent = GH.tpl.board.x.renderDateView({field: field});
            break;
        case 'text':
        default:
            fieldContent = GH.tpl.board.x.renderTextView({field: field});
            break;
	}
	return fieldContent;
};


//
// Field editing
//

/**
 * Focuses the field
 */
GH.RapidBoard.DetailFields.focusEditedField = function(fieldId) {
	var editInfo = GH.RapidBoard.DetailFields.getEditInfo(fieldId);
	if (editInfo) {
		editInfo.editor.focus(editInfo);
	}
};

/**
 * Opens a field in edit mode.
 */
GH.RapidBoard.DetailFields.editField = function(editData, fieldData) {
	// put together a data carrier object, which holds the info throughout the edit cycle
	var editInfo = {};
	editInfo.editData = editData;
	editInfo.fieldData = fieldData;
	editInfo.enabled = true;
	
	// find the correct view dd for the edit
	var viewDd = AJS.$('dd.ghx-fieldname-'+fieldData.id);
	if (viewDd.length < 1) return;
	
	// add view and edit dd
	editInfo.viewDd = viewDd;
	editInfo.editDd = GH.RapidBoard.DetailFields.createEditDD(fieldData);
	
	// find correct editor for the field
	if (fieldData.type == 'textarea') {
		editInfo.editor = new GH.RapidBoard.DetailFields.TextArea();
	} else if (fieldData.type == 'text') {
		editInfo.editor = new GH.RapidBoard.DetailFields.TextInput();
	}

	// start editing if we got a valid editInfo object.
	if (editInfo.editor) {
		GH.RapidBoard.DetailFields.addEditInfo(editInfo);
		editInfo.editor.startEdit(editInfo);
	}
};

/**
 * Creates the edit dd dom
 */
GH.RapidBoard.DetailFields.createEditDD = function(fieldData) {
	var elem = AJS.$('<dd></dd>');
	elem.addClass('ghx-fieldtype-'+fieldData.type)
		.addClass('ghx-fieldname-'+fieldData.id).
		attr({'data-field-id': fieldData.id});
	return elem;
};

/** Switches the view for the edit. */
GH.RapidBoard.DetailFields.showEdit = function(editInfo) {
	// hide view and insert edit after view
	editInfo.viewDd.hide().after(editInfo.editDd);
};

/**
 * Removes the edit and shows the previous view again.
 */
GH.RapidBoard.DetailFields.hideEdit = function(editInfo) {
	// remove edit, then show view
	editInfo.editDd.remove();
	editInfo.viewDd.show();
	
	// remove the edit info as it is no longer valid
	GH.RapidBoard.DetailFields.removeEditInfo(editInfo);
};

/**
 * Displays an error for a given field.
 */
GH.RapidBoard.DetailFields.showFieldErrors = function(fieldId, errors) {
	var editInfo = GH.RapidBoard.DetailFields.getEditInfo(fieldId);
	if (editInfo) {
		// show the errors
		editInfo.editor.setErrors(editInfo, errors);
		// re-enable the field
		editInfo.editor.returnToEdit(editInfo);
	} else {
		// unknown field or filed removed already
		AJS.log('Unable to find field for id ' + fieldId);
		GH.RapidBoard.Notification.showErrors({ errors : errors});
	}
};

/**
 * Defining the base class for an Edit Control (TextArea or TextInput)
 */
GH.RapidBoard.DetailFields.EditControl = function(){};

GH.RapidBoard.DetailFields.EditControl.prototype = {

    generateControl : function(editInfo) {
        // implement this
        throw 'Unimplemented method';
    },

    keydownHandler : function(event, editInfo) {
        var self = this;
        if (event.keyCode == 27) { // escape key
            event.stopPropagation();
            self.cancelEdit(editInfo);
        }
        else if (event.keyCode == 13) { // return
            event.preventDefault();
            self.submitChanges(editInfo);
        }
        else if (event.keyCode == 9) { // tab
            // 1: tab, -1: shift-tab
            var delta = event.shiftKey ? -1 : 1;
            var success = GH.RapidBoard.DetailsView.editRelativeField(editInfo.fieldData.id, delta);
            if (success) {
                // prevent default
                //event.preventDefault();
                // trigger submitting the changes - the blur event is not properly fired in IE
                //GH.RapidBoard.DetailFields.TextArea.submitChanges(editInfo);
            }
        }
    },

    /**
     * Called to initiate editing using the provided editInfo
     */
    startEdit : function(editInfo) {
        var self = this;

        var editControl = self.generateControl(editInfo);

        // set the value inside the textarea
        editControl.val(editInfo.fieldData.text ? editInfo.fieldData.text : '');

        // On blur return the field with new value
        editControl.bind('blur focusout', function() {
            self.submitChanges(editInfo);
        });

        // handle some keys specially
        editControl.keydown(function(event) {
            self.keydownHandler(event, editInfo);
        });

        // show the edit and focus the area
        GH.RapidBoard.DetailFields.showEdit(editInfo);

        // focus the text area
        self.focus(editInfo);

        // mark the field as enabled now
        editInfo.enabled = true;
    },

    /**
     * Cancels the edit operation
     */
    cancelEdit : function(editInfo) {
        // disable
        if (!editInfo.enabled) return;
        editInfo.enabled = false;

        // remove blur handler from textarea
        editInfo.editControl.unbind('blur');

        // then switch out the edit area
        GH.RapidBoard.DetailFields.hideEdit(editInfo);
    },

    /**
     * Submits a new value for the given edit field
     */
    submitChanges : function(editInfo) {
        var self = this;
        // disable, ensure only one event can trigger the submit
        if (!editInfo.enabled) return;
        editInfo.enabled = false;

        // check whether we actually changed, otherwise just cancel the edit
        var newValue = editInfo.editControl.val();
        if (newValue == editInfo.fieldData.text) {
            editInfo.enabled = true;
            self.cancelEdit(editInfo);
            return;
        }

        // change the ui into a processing state...
        editInfo.editControl.unbind('blur').attr("disabled", "disabled");
        editInfo.editDd.find('.gh-spinner').show();

        // run the validators

        // trigger the submitting of the value
        AJS.$.ajax({
            url: GH.Ajax.buildRestUrl('/xboard/issue/update-field.json'),
            type: 'PUT',
            data: {
                'issueId' : editInfo.editData.id,
                'fieldId' : editInfo.fieldData.id,
                'newValue' : newValue
            },
            dataType: 'json',
            success: function(data) {
                if (data.success) {
                    GH.RapidBoard.DetailsView.handleFieldUpdateSuccess(data.success);
                } else {
                    GH.RapidBoard.DetailsView.handleFieldUpdateFailure(editInfo.editData.id, editInfo.fieldData.id, data.error);
                }
                // analytics
                AJS.$(AJS).trigger("gh.issuedetail.inlineedit", {
                    fieldId : editInfo.fieldData.id
                });
            }
        });
    },

    setErrors : function(editInfo, errors) {
        // TODO: render using template
        var text = '';
        for (var i=0; i < errors.length; i++) {
            if (i > 0) text += '<br>';
            text += errors[i].message;
        }
        editInfo.editDd.addClass('ghx-has-error').find('.js-fielderrors').text(text);
    },

    /**
     * Called in case of a submit or validation error
     */
    returnToEdit : function(editInfo) {
        var self = this;
        // make sure we add the blur back in
        editInfo.editControl.blur(function(){
            self.submitChanges(editInfo);
        });

        // remove the spinner
        editInfo.editDd.find('.gh-spinner').hide();

        // reenable the area
        editInfo.editControl.removeAttr('disabled');

        // ask for the focus back, but only if there are no other elements being edited (happens in case of tabbing through the elements)
        if (GH.RapidBoard.DetailFields.getEditingCount() == 1) {
            self.focus(editInfo);
        }

        // re-enable
        editInfo.enabled = true;
    },

    /**
     * Focus the edited field
     */
    focus : function(editInfo) {
        var focus = function() {
            editInfo.editControl.focus();
        };
        setTimeout(focus, 0);
    }
};

/**
 * TextArea edit support
 */
GH.RapidBoard.DetailFields.TextArea = function () {};
GH.RapidBoard.DetailFields.TextArea.prototype = new GH.RapidBoard.DetailFields.EditControl();

GH.RapidBoard.DetailFields.TextArea.prototype.generateControl = function(editInfo) {
    // render the edit html and insert it into the editDd
    var fieldContent = GH.tpl.board.x.renderTextareaEdit({field: editInfo.fieldData});
    editInfo.editDd.html(fieldContent);

    // append listeners
    var textarea = editInfo.editDd.find('textarea');
    editInfo.editControl = textarea;
    return textarea;
};

GH.RapidBoard.DetailFields.TextArea.prototype.keydownHandler = function(event, editInfo) {
    var self = this;
    if (event.keyCode == 27) { // escape key
        event.stopPropagation();
        self.cancelEdit(editInfo);
    }
    else if (event.keyCode == 13) { // return
        // check if shift is pressed: return = form submit, shift return = new line
        if (event.shiftKey != true) {
            event.preventDefault();
            self.submitChanges(editInfo);
        }
    }
    else if (event.keyCode == 9) { // tab
        // 1: tab, -1: shift-tab
        var delta = event.shiftKey ? -1 : 1;
        var success = GH.RapidBoard.DetailsView.editRelativeField(editInfo.fieldData.id, delta);
        if (success) {
            // prevent default
            //event.preventDefault();
            // trigger submitting the changes - the blur event is not properly fired in IE
            //GH.RapidBoard.DetailFields.TextArea.submitChanges(editInfo);
        }
    }
};

/**
 * Text edit support
 */
GH.RapidBoard.DetailFields.TextInput = function () {};
GH.RapidBoard.DetailFields.TextInput.prototype = new GH.RapidBoard.DetailFields.EditControl();

GH.RapidBoard.DetailFields.TextInput.prototype.generateControl = function(editInfo) {
    // render the edit html and insert it into the editDd
    var fieldContent = GH.tpl.board.x.renderTextEdit({field: editInfo.fieldData});
    editInfo.editDd.html(fieldContent);

    // append listeners
    var textControl = editInfo.editDd.find('input');
    editInfo.editControl = textControl;
    return textControl;
};

