/**
 * Issue functions
 * 
 * Handles issue functionality
 */


/** Renders the columns for the given model. */
GH.RapidBoard.Issue = {};

/**
* Holds the coordinates of the most recently selected issue.
*/
GH.RapidBoard.Issue.lastSelectedIssueLocation = {};

/**
* Holds the selected issues in form of a stack, thus most recently selected issues are pushed onto the top of the stack.
*/
GH.RapidBoard.Issue.selectedIssueStack = [];

/**
 * Get the selected issues in form of an array of issue keys. The order is according to the insertion order.
 */
GH.RapidBoard.Issue.getSelectedIssues = function() {
    return GH.RapidBoard.Issue.selectedIssueStack;
};

/**
 * Get the selected issues in order of issue "rank"
 */
GH.RapidBoard.Issue.getSelectedIssuesInOrder = function() {
    // sort by visibility index, which is returned directly by getIssueVisibilityIndexByKey
    return _.sortBy(GH.RapidBoard.Issue.selectedIssueStack, GH.RapidBoard.RapidListDataController.getIssueVisibilityIndexByKey);
};

/**
 * Is the passed issue key currently in the selection
 */
GH.RapidBoard.Issue.isInSelection = function(issueKey) {
    return _.contains(GH.RapidBoard.Issue.selectedIssueStack, issueKey);
};

/**
 * Can the issueKey be added to the selection.
 */
GH.RapidBoard.Issue.canAddToSelection = function(issueKey) {
    // if there are no issues, adding is always allowed
    if (GH.RapidBoard.Issue.selectedIssueStack.length == 0) return true;
    
    // ensure the issue is compatible with the current selection
    var firstKey = GH.RapidBoard.Issue.selectedIssueStack[0];
    return GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(firstKey, issueKey);
};

/**
 * Add an issue key to the selection
 */
GH.RapidBoard.Issue._addSelectedIssueKey = function(issueKey) {
	// only add if we are not in the selection already
	if(!GH.RapidBoard.Issue.isInSelection(issueKey)) {
        GH.RapidBoard.Issue.selectedIssueStack.push(issueKey);
    }
};

/**
 * Remove an issue key from the selection
 */
GH.RapidBoard.Issue._removeSelectedIssueKey = function(issueKey) {
	GH.RapidBoard.Issue.selectedIssueStack = _.without(GH.RapidBoard.Issue.selectedIssueStack, issueKey);
};

/**
 * Adds a list of issues to the current selection
 */
GH.RapidBoard.Issue.addSelectedIssueKeys = function(issueKeys, updateCurrentSelection) {
	// add each issue, retaining whether the issue got actually added
    _.each(issueKeys, function(issueKey){
    	GH.RapidBoard.Issue._addSelectedIssueKey(issueKey);
    });
    
    // update current selection 
    if (updateCurrentSelection) {
    	GH.RapidBoard.Issue.updateSelectedStackIssue();
    }
};

/**
 * Removes a list of issues from the current selection
 */  
GH.RapidBoard.Issue.removeSelectedIssueKeys = function(issueKeys, updateCurrentSelection) {
	// remove each issue
    _.each(issueKeys, function(issueKey){
    	GH.RapidBoard.Issue._removeSelectedIssueKey(issueKey);
    });
    
    // update current selection
    if (updateCurrentSelection) {
    	GH.RapidBoard.Issue.updateSelectedStackIssue();
    }
};

/**
 * Sets the list of issues in the current selection.
 */  
GH.RapidBoard.Issue.replaceSelectedIssueKeys = function(issueKeys, updateCurrentSelection) {
	GH.RapidBoard.Issue.clearSelectedIssueKeys(false);
    GH.RapidBoard.Issue.addSelectedIssueKeys(issueKeys, updateCurrentSelection);
};

/**
 * Clears the list of currently selected issues
 */
GH.RapidBoard.Issue.clearSelectedIssueKeys = function(updateCurrentSelection) {
    GH.RapidBoard.Issue.selectedIssueStack = [];
    
    // update current selection
    if (updateCurrentSelection) {
    	GH.RapidBoard.Issue.updateSelectedStackIssue();
    }
};

/**
 * Updates the selected issue according to the current issue stack.
 */
GH.RapidBoard.Issue.updateSelectedStackIssue = function() {
    // fetch selected issue, will be undefined in case on an empty stack
    var selectedIssue = GH.RapidBoard.Issue.selectedIssueStack[GH.RapidBoard.Issue.selectedIssueStack.length - 1];

    // update the selection, will update the url and detail view. handles undefined as unselect
    GH.RapidBoard.Issue._selectIssue(selectedIssue, {scrollToView: false});
};

/**
 * Refreshes the model, setting the the selected state on all issues.
 * This function is called after model replacements which won't contain this information
 */ 
GH.RapidBoard.Issue.refreshSelectionModel = function() {
	// fetch the current selected issue id
	var selectedIssueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	
	// update the selection model and see whether the issue is indeed valid
	var selectionValid = GH.RapidBoard.Issue.updateSelectionModel(selectedIssueKey);
	
	// multi selection support
    if(selectionValid) {
        // if we got a valid selected key, set it as the selected one
    	GH.RapidBoard.Issue.replaceSelectedIssueKeys([selectedIssueKey], false);
    } else {
        // else clear the selected issues
    	GH.RapidBoard.Issue.clearSelectedIssueKeys(false);
    }
	
	// issue not valid, clear out the url state in case of invalid selected issue
	if (!selectionValid && selectedIssueKey) {
		GH.RapidBoard.State.setSelectedIssueKey(null);
		GH.RapidBoard.State.replaceState(); // no push here, as current state is invalid
	}
};

/**
 * Updates the model state with the selected issue information.
 * @return true if the issue could be located, false otherwise
 */
GH.RapidBoard.Issue.updateSelectionModel = function(issueKey) {
	var idValid = false;
	var callback = function(data) {
		if (data.key == issueKey && !data.isFiltered) {
			data.selected = true;
			idValid = true;
		} else {
			data.selected = false;
		}
	};
    GH.RapidBoard.RapidListDataController.applyToIssues(callback);
	return idValid;
};

/**
 * Get the selected issue id, if none is found and fallbackToLastIssueLocation is true
 * then return the issue at the location of the last selected issue
 */
GH.RapidBoard.Issue.getSelectedIssueKey = function(fallbackToLastIssueLocation) {
    // fetch the selected key
    var issueKey = GH.RapidBoard.State.getSelectedIssueKey();

    // check whether the selected issue key is actually valid
    if (issueKey) {
        var issueData = GH.RapidBoard.RapidListDataController.getIssueDataByKey(issueKey);
        if (issueData && issueData.key) {
            return issueData.key;
        }
    }

    // if we don't have a valid selection, try to select an issue at the last known location
    if (fallbackToLastIssueLocation) {
        var issueData = GH.RapidBoard.RapidListDataController.getIssueDataAtLocation(GH.RapidBoard.Issue.lastSelectedIssueLocation);
        if (issueData && issueData.key) {
            issueKey = issueData.key;
            // update the selection
            GH.RapidBoard.Issue.selectIssue(issueKey, {pushState:false});
            GH.RapidBoard.State.replaceState();
            return issueKey;
        }
    }

    return false;
};

GH.RapidBoard.Issue.getSelectedIssueId = function() {
    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    if (issueKey) {
        var issueData = GH.RapidBoard.RapidListDataController.getIssueDataByKey(issueKey);
        if (issueData && issueData.id) {
            return issueData.id;
        }
    }
};

/**
 * Go to the given issue.
 * This will select the issue if not yet selected, otherwise simply scroll the issue into view
 */
GH.RapidBoard.Issue.gotoIssue = function(issueKey) {
	if (!issueKey) return;
	var selectedKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	if (issueKey != selectedKey) {
		GH.RapidBoard.Issue.selectIssue(issueKey);
	} else {
		var elem = AJS.$('.ghx-columns').find('.ghx-issue[data-issue-key="' + issueKey + '"]');
		GH.RapidBoard.Issue._scrollToView(elem);
	}
};

GH.RapidBoard.Issue.selectIssue = function(issueKey, options) {
	GH.RapidBoard.Issue.replaceSelectedIssueKeys([issueKey], false);
    GH.RapidBoard.Issue._selectIssue(issueKey, options);
};

/**
 * Selects an issue given its index.
 */
GH.RapidBoard.Issue._selectIssue = function(issueKey, options) {
    var defaultOptions = {
        pushState: true,
        scrollToView: true
    };
    options = _.extend(defaultOptions, options);
	
	// update the model
	var isIssueValid = GH.RapidBoard.Issue.updateSelectionModel(issueKey);
	
	// update the state
	if (isIssueValid) {
	    GH.RapidBoard.State.setSelectedIssueKey(issueKey);
        GH.RapidBoard.Issue.lastSelectedIssueLocation = GH.RapidBoard.RapidListDataController.getIssuePositionByKey(issueKey)
	} else {
		GH.RapidBoard.State.setSelectedIssueKey(null);
        GH.RapidBoard.Issue.lastSelectedIssueLocation = {};
	}
	if (options.pushState) {
		GH.RapidBoard.State.pushState();
	}

    // update the ui
    GH.RapidBoard.Issue._updateSelectionUI(options.scrollToView ? issueKey : null);
};

GH.RapidBoard.Issue._updateSelectionUI = function(scrollToIssueKey) {
    var scrollToViewElem = null;
    AJS.$('.ghx-issue').removeClass('ghx-selected').each(function(index, elem) {
        var $elem = AJS.$(elem);
        var issueKey = $elem.data('issue-key');
        if (_.contains(GH.RapidBoard.Issue.selectedIssueStack, issueKey)) {
            $elem.addClass('ghx-selected');
        }
        if (issueKey == scrollToIssueKey) {
            scrollToViewElem = $elem;
        }
    });

    // scroll the currently selected issue to view
    if (scrollToViewElem) {
        GH.RapidBoard.Issue._scrollToView(scrollToViewElem);
    }

    // fire an event to inform others that the selection changed
    var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
    var issueData = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
    AJS.$(document).trigger('issueSelected', issueData ? issueData.issue.key : undefined);
};

/**
 * Ensures the passed issue element is fully visible
 * PERF This function is super-expensive. Optimizing is non-trivial
 */
GH.RapidBoard.Issue._scrollToView = function($elem) {
    // make sure the issue is properly visible
    var issueHeight = $elem.height() + 10;
    var options = {
        marginBottom: issueHeight,
        marginTop: issueHeight
    };
    $elem.scrollIntoView(options);
};

/**
 * Get the id of an issue given its element
 */
GH.RapidBoard.Issue.getIssueId = function(elem) {
    return elem.closest('.ghx-issue, .ghx-issue-filtered, .ghx-parent-group').data('issue-id');
};
GH.RapidBoard.Issue.getIssueKey = function(elem) {
    return elem.closest('.ghx-issue, .ghx-issue-filtered, .ghx-parent-group').data('issue-key');
};

/**
 * Loads the data for an issue, then updates the column model and view
 */
GH.RapidBoard.Issue.loadIssue = function(event, issueId) {
	AJS.$.ajax({
        url: GH.Ajax.buildRestUrl('/xboard/columnissue.json'),
        type: 'GET',
        data: {
        	'rapidViewId' : GH.RapidBoard.State.getRapidViewId(),
            'issueId' : issueId
        },
        dataType: 'json',
        success: GH.RapidBoard.Issue.updateIssue
    });
};


/**
 * Update the data for an issue, then re-render
 */
GH.RapidBoard.Issue.updateIssue = function(data) {
	// update the data model
	var issueData = GH.RapidBoard.RapidListDataController.updateIssueData(data.success);
	
	// then re-render that issue
	var issue = AJS.$('.ghx-issue[data-issue-key="' + issueData.key + '"]');
	if (issue.length > 0) {
		// render the issue
		var issueDom = AJS.$(GH.tpl.rapid.swimlane.renderIssue({issue: issueData}));
		issue.replaceWith(issueDom);
	}
};

/**
 * Holds the issues that got clicked on without the shift key being pressed.
 */
GH.RapidBoard.Issue.lastNonShiftSelectedKey = undefined;
/**
 * Holds the issues that got clicked on with the shift key being pressed.
 */
GH.RapidBoard.Issue.lastShiftSelectedKey = undefined;

/**
 * Get the last non shift issue key.
 */
GH.RapidBoard.Issue.getLastNonShiftSelectedKey = function() {
	if (!GH.RapidBoard.Issue.lastNonShiftSelectedKey) {
		// see whether we got a selection, use that key instead
		var key = GH.RapidBoard.Issue.getSelectedIssueKey();
		if (key) {
			GH.RapidBoard.Issue.setLastNonShiftSelectedKey(key);
			return key;
		}
	}
	return GH.RapidBoard.Issue.lastNonShiftSelectedKey;
};

/**
 * Set the last non shift selected issue key
 */
GH.RapidBoard.Issue.setLastNonShiftSelectedKey = function(issueKey) {
	GH.RapidBoard.Issue.lastNonShiftSelectedKey = issueKey;
	GH.RapidBoard.Issue.lastShiftSelectedKey = undefined;
};

/**
 * Handles issue click events
 */
GH.RapidBoard.Issue.handleIssueClick = function(event) {
    // fetch the key for the given issue
    var issueKey = GH.RapidBoard.Issue.getIssueKey(AJS.$(this));
    if(!issueKey) {
        return;
    }

    var metaDown = event.metaKey;
    var shiftDown = event.shiftKey;
    
    // easiest case, neither shift nor ctrl down
    if (!metaDown && !shiftDown) {
        GH.RapidBoard.Issue.selectIssue(issueKey);
    }
    // stop doing any work if the issueKey cannot be added to the selection
    else if (!GH.RapidBoard.Issue.canAddToSelection(issueKey)) {
        return;
    }
    
    // slightly more complicated, ctrl without shift
    else if (metaDown && !shiftDown) {
    	// toggled selection of clicked already selected
        if(GH.RapidBoard.Issue.isInSelection(issueKey)) {
            GH.RapidBoard.Issue.removeSelectedIssueKeys([issueKey], true);
        }
    	// ensure we can actually add the issue to the selection
        else if (GH.RapidBoard.Issue.canAddToSelection(issueKey)) {
    		GH.RapidBoard.Issue.addSelectedIssueKeys([issueKey], true);
        }
    }
    
    // shift, mother of all selections
    else {
        // fetch the last selected key that was a non-shift selection
        var lastClickedIssueKey = GH.RapidBoard.Issue.getLastNonShiftSelectedKey();
        
        // if we have no selected issues, or we don't have a last non-shift key simply select the issue 
        if (GH.RapidBoard.Issue.selectedIssueStack.length == 0 || !lastClickedIssueKey) {
        	GH.RapidBoard.Issue.selectIssue(issueKey);
        	GH.RapidBoard.Issue.setLastNonShiftSelectedKey(issueKey);
        }

        else {
        	// calculate the issues between the current and the recent selection
        	var issueRangeToAdd = GH.RapidBoard.RapidListDataController.getRankableIssuesRange(lastClickedIssueKey, issueKey);
            if(metaDown) {
                // also calculate the old range, as we have to remove issues added in the previous shift selection
                var issueRangeToRemove = [];
                if(GH.RapidBoard.Issue.lastShiftSelectedKey) {
                    issueRangeToRemove = GH.RapidBoard.RapidListDataController.getRankableIssuesRange(lastClickedIssueKey, GH.RapidBoard.Issue.lastShiftSelectedKey) || [];

                    // ensure no new issues are contained in the issues to remove
                    issueRangeToRemove = _.without(issueRangeToRemove, issueRangeToAdd);
                }

	        	// remove the old range and add the new range to the selection
	        	GH.RapidBoard.Issue.removeSelectedIssueKeys(issueRangeToRemove, false);
	        	GH.RapidBoard.Issue.addSelectedIssueKeys(issueRangeToAdd, true);
	        } else {
		        // replace whatever is currently selected with the new selection
	        	GH.RapidBoard.Issue.replaceSelectedIssueKeys(issueRangeToAdd, true);
	        }
        }
    }
    
    // retain the issueKey if we are the last non-shift clicked issue
    if (!shiftDown) {
    	GH.RapidBoard.Issue.setLastNonShiftSelectedKey(issueKey);
    } else {
    	GH.RapidBoard.Issue.lastShiftSelectedKey = issueKey;
    }
    
    // work around IE8 text selection
    GH.RapidBoard.Issue.clearTextSelection();
};

/**
 * Hack workaround to deselect text in IE8 as a result of shift/ctrl click events
 */
GH.RapidBoard.Issue.clearTextSelection = function() {
	// performs the actual deselection
	var clearSelection = function() {
	  if(document.selection && document.selection.empty) {
    		document.selection.empty();
    	} else if(window.getSelection) {
        	var selection = window.getSelection();
        	selection.removeAllRanges();
    	}
    };
    
    // unselect needs to happen after the click event, so we defer
    setTimeout(clearSelection, 0);
};

/**
 * Called upon page initialization
 */
GH.RapidBoard.Issue.init = function() {    
    // issue selection
    AJS.$('#ghx-pool').delegate('.ghx-issue','click', GH.RapidBoard.Issue.handleIssueClick);

    // listen to updates to an issue.
    AJS.$(document).bind('issueUpdated', GH.RapidBoard.Issue.loadIssue);
};
