/**
 * Controller for the rapid list data
 *
 * Provides access to columns, swimlanes and individual issues.
 */
GH.RapidBoard.RapidListDataController = {};

GH.RapidBoard.RapidListDataController.MS_DAY = 1000*60*60*24;

/**
 * called to initialize the data model
 */
GH.RapidBoard.RapidListDataController.initModel = function() {
	if (GH.view.model.rapidList && GH.view.model.rapidList.success) {
		GH.view.model.rapidList.success = GH.RapidBoard.RapidListDataController.preprocessData(GH.view.model.rapidList.success);

	    // update the selected issue state
		GH.RapidBoard.Issue.refreshSelectionModel();
	}
};


/**
 * Get the rapid list data - returns false in case the data has not been loaded
 */
GH.RapidBoard.RapidListDataController.getRapidListData = function() {
	if (GH.view.model.rapidList && GH.view.model.rapidList.success) return GH.view.model.rapidList.success;
	return false;
};

GH.RapidBoard.RapidListDataController.getRapidListErrors = function() {
	if (GH.view.model.rapidList && GH.view.model.rapidList.error) return GH.view.model.rapidList.error.errors;
	return false;
};

/**
 * Preprocesses certain model data information, e.g. the issue count
 */
GH.RapidBoard.RapidListDataController.preprocessData = function(rapidList) {

    // go count issues
    GH.RapidBoard.RapidListDataController.computeIssueCount(rapidList);

    // evaluate column limits, if there's a statistics field set
    if (rapidList.statistics.fieldConfigured) {
        GH.RapidBoard.RapidListDataController.computeBusted(rapidList);
    }
    
    GH.RapidBoard.RapidListDataController.computeTimeInColumn();

    return rapidList;
};

/**
 * compute time spent in current column. The server gives us the total duration in milliseconds,
 * at the moment we hardcode that to days (24h). Rounding is always down.
 */
GH.RapidBoard.RapidListDataController.computeTimeInColumn = function() {
    GH.RapidBoard.RapidListDataController.applyToIssues(
    		GH.RapidBoard.RapidListDataController.computeTimeInColumnForIssue
    );
};

GH.RapidBoard.RapidListDataController.computeTimeInColumnForIssue = function(issue) {
	issue.daysInColumn = Math.floor(issue.timeInColumn / GH.RapidBoard.RapidListDataController.MS_DAY);
};

/**
 * Compute the issue count data for columns, swimlanes and totals
 */
GH.RapidBoard.RapidListDataController.computeIssueCount = function(rapidList) {
    var swimlanes = rapidList.swimlanes;
    var totalIssueCount = 0, totalVisibleIssueCount = 0;
    for (var h=0; h < swimlanes.length; h++) {
        var swimlane = swimlanes[h];
        var columns = swimlane.columns;
        var swimlaneIssueCount = 0, swimlaneVisibleIssueCount = 0;
        for (var i=0; i < columns.length; i++) {
        	var subTaskIssue = null, previousIssue = null;
            var listColumn = rapidList.columns[i];
            if (!listColumn.issueCount) listColumn.issueCount = 0;
            if (!listColumn.visibleIssueCount) listColumn.visibleIssueCount = 0;
            var column = columns[i];
            var issues = column.issues || [];
            column.issueCount = issues.length;
            column.visibleIssueCount = 0;
            for(var j = 0; j < issues.length; j++) {
            	var issue = issues[j];
                if(!issue.isFiltered) {
                    column.visibleIssueCount++;
                    
                    // Set the beginGroup, fakeParent and endGroup flags, used to render parent/subtask groups
                    if (issue.parentId) {

                    	// only set first if we are currently not in a subtask case already
                    	if (! subTaskIssue) {
                    		if (previousIssue && previousIssue.id == issue.parentId) { // parent is available, mark it as container
                    			previousIssue.beginGroup = true;
                    		} else {
                    			// no parent available, insert placeholder parent
                    			issue.fakeParent = true;
                    		}
                    		issue.beginGroup = true;
                    	} else if (subTaskIssue.parentId != issue.parentId) {
                    		subTaskIssue.endGroup = true; // close previous subtask
                    		issue.beginGroup = true;
                    		issue.fakeParent = true;
                    	}
                    	subTaskIssue = issue;
                    } else {
                    	if (subTaskIssue) {
                    		subTaskIssue.endGroup = true;
                    		subTaskIssue = null;
                    	}
                    }
                    previousIssue = issue;
                }
            }
            if (subTaskIssue) {
        		subTaskIssue.lastSubTask = true;
        		subTaskIssue = null;
        	}
            swimlaneIssueCount += column.issueCount;
            swimlaneVisibleIssueCount += column.visibleIssueCount;
            listColumn.issueCount += column.issueCount;
            listColumn.visibleIssueCount += column.visibleIssueCount;
        }
        swimlane.issueCount = swimlaneIssueCount;
        swimlane.visibleIssueCount = swimlaneVisibleIssueCount;
        totalIssueCount += swimlaneIssueCount;
        totalVisibleIssueCount += swimlaneVisibleIssueCount;
    }

    rapidList.issueCount = totalIssueCount;
    rapidList.visibleIssueCount = totalVisibleIssueCount;
};

/**
 * Compute information about column busted state, based on the statistics limits
 */
GH.RapidBoard.RapidListDataController.computeBusted = function(rapidList) {
    // min/max busted for columns
    rapidList.anyColumnBusted = false;
    for (var i=0; i<rapidList.columns.length; i++) {
        // TODO make more general for different types of constraints
        var column = rapidList.columns[i];
        column.minBusted = (typeof column.min != 'undefined') && column.min > column.issueCount;
        column.maxBusted = (typeof column.max != 'undefined') && column.max < column.issueCount;

        // any column busted
        rapidList.anyColumnBusted = rapidList.anyColumnBusted || column.minBusted || column.maxBusted;
    }
};

/**
 * Get all swimlanes
 */
GH.RapidBoard.RapidListDataController.getAllSwimlanes = function() {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.swimlanes : [];
};

/**
 * Get all visible swimlanes
 */
GH.RapidBoard.RapidListDataController.getAllVisibleSwimlanes = function() {
    var visualSwimlanes = [], swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();

	for (var i=0; i < swimlanes.length; i++) {
		if (swimlanes[i].visibleIssueCount > 0) {
            var swimlane = AJS.$.extend(true, {}, swimlanes[i]);
            // set whether the swimlane is collapsed, only valid if we have swimlanes
            swimlane.collapsed = hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(swimlane.id);
            visualSwimlanes.push(swimlane);
        }
	}

	visualSwimlanes.hasSwimlanes = hasSwimlanes;
    return visualSwimlanes;
};

GH.RapidBoard.RapidListDataController.hasSwimlanes = function() {
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    for (var i=0; i < swimlanes.length; i++) {
		var swimlane = swimlanes[i];

		if (swimlane.visibleIssueCount > 0 && !swimlane.defaultSwimlane){
           return true;
		}
	}
    return false;
};

/**
 * Get a swimlane given its index
 */
GH.RapidBoard.RapidListDataController.getSwimlane = function(index) {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.swimlanes[index] : undefined;
};

/**
 * Get all quick filters
 */
GH.RapidBoard.RapidListDataController.getAllQuickFilters = function() {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.quickFilters : [];
};

/**
 * Get a quick filter given its index
 */
GH.RapidBoard.RapidListDataController.getQuickFilter = function(index) {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.quickFilters[index] : undefined;
};

/**
 * Get the statistics field info
 */
GH.RapidBoard.RapidListDataController.getStatistics = function() {
    var data = GH.RapidBoard.RapidListDataController.getRapidListData();
    return data ? data.statistics : undefined;
};

/**
 * Get all columns
 */
GH.RapidBoard.RapidListDataController.getAllColumns = function() {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.columns : [];
};

/**
 * Get a column given its index
 * Note that this returns the column object which contains status mappings and column name, but no issues
 */
GH.RapidBoard.RapidListDataController.getColumn = function(index) {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return data ? data.columns[index] : undefined;
};

/**
 * Get the number of columns in the model.
 */
GH.RapidBoard.RapidListDataController.getColumnCount = function() {
	var data = GH.RapidBoard.RapidListDataController.getRapidListData();
	return ((data && data.columns) ? data.columns : []).length;
};

/**
 * Get the issue column id for a given dom element
 */
GH.RapidBoard.RapidListDataController.getColumnId = function(columnElem) {
    return AJS.$(columnElem).closest('.ghx-column').attr('data-column-id');
};


// Issue positioning functions
/**
 * Updates the model with new data for a single issue
 */
GH.RapidBoard.RapidListDataController.updateIssueData = function(issueData) {
	// preprocess
	GH.RapidBoard.RapidListDataController.computeTimeInColumnForIssue(issueData);
	
	// replace
	var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    for (var h=0; h < swimlanes.length; h++) {
    	var swimlane = swimlanes[h];
		for (var i=0; i < swimlane.columns.length; i++) {
			var column = swimlane.columns[i];
			var issues = column.issues || [];
			for (var j=0; j < issues.length; j++) {
				if (issues[j].id == issueData.id) {
					// TODO: this should be handled by a callback and not in here
					issueData.selected = issues[j].selected;
					issueData.fakeParent = issues[j].fakeParent;
					issueData.beginGroup = issues[j].beginGroup;
					issueData.endGroup = issues[j].endGroup;
					issueData.lastSubTask = issues[j].lastSubTask;
					issues[j] = issueData;
				}
			}
		}
	}
    return issueData;
};

/**
 * Applies a function to all issues, replacing the data object if returned by the callback
 */
GH.RapidBoard.RapidListDataController.applyToIssues = function(callback) {
	var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    for (var h=0; h < swimlanes.length; h++) {
    	var swimlane = swimlanes[h];
		for (var i=0; i < swimlane.columns.length; i++) {
			var column = swimlane.columns[i];
			var issues = column.issues || [];
			for (var j=0; j < issues.length; j++) {
				callback(issues[j]);
			}
		}
	}
};

/**
 * Return position data for a given issue
 *
 * Note: this is an internal function that should not be called outside of this controller
 */
GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey = function(issueKey) {
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    var swimlaneVisibilityIndex = 0, issueVisibilityIndex = 0;

    for (var h=0; h < swimlanes.length; h++) {
    	var swimlane = swimlanes[h];
        for (var i=0; i < swimlane.columns.length; i++) {
            var column = swimlane.columns[i];
            var issues = column.issues || [];
            for (var j=0; j < issues.length; j++) {

                if (issues[j].key == issueKey) {
                    return {
                        swimlane: swimlane,
                        swimlaneIndex: h,
                        swimlaneVisibilityIndex: swimlaneVisibilityIndex,
                        column: column,
                        columnIndex: i,
                        issueIndex: j,
                        issueVisibilityIndex: issueVisibilityIndex,
                        issue: issues[j]
                    };
                }
                if(!issues[j].isFiltered) {
                    issueVisibilityIndex++;
                }
            }
            column.issues = issues;
            issueVisibilityIndex = 0;
        }
        if(swimlane.visibleIssueCount > 0){
            swimlaneVisibilityIndex++;
        }
	}
    return false;
};

/**
 * get the position (as a combination of swimlane, column and visibility index within that column), for
 * the issue with key issueKey
 */
GH.RapidBoard.RapidListDataController.getIssuePositionByKey = function(issueKey) {
    var issueData = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
    if (!issueData) return undefined;
    return {
        swimlaneId: issueData.swimlane.id,
        columnId: issueData.column.id,
        position: issueData.issueVisibilityIndex
    };
};

/**
 * Get the swimlane index a given issue is currently in
 */
GH.RapidBoard.RapidListDataController.getSwimlaneIndexByKey = function(issueKey) {
	var data = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	return data ? data.swimlaneIndex : false;
};

/**
 * Get the column index a given issue is currently in
 */
GH.RapidBoard.RapidListDataController.getColumnIndexByKey = function(issueKey) {
	var data = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	return data ? data.columnIndex : false;
};

/**
 * Get the visibility index of an issue within a column
 */
GH.RapidBoard.RapidListDataController.getIssueVisibilityIndexByKey = function(issueKey) {
	var data = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	return data ? data.issueVisibilityIndex : false;
};

/**
 * Finds the next issue to select given a set of to-be-ranked issues
 */
GH.RapidBoard.RapidListDataController.getBestFitSelectionAfterRank = function(toBeRankedIssueKeys) {
    // the issue keys are in order, so we really only care about the last one
    var lastSelectedIssueKey = _.last(toBeRankedIssueKeys);
    return GH.RapidBoard.RapidListDataController.getNextIssueAfterSendToBottom(lastSelectedIssueKey);
};

/**
 * Returns the id of the issue to go to after sending the specified issue to the bottom
 * We have to do some tricky handling to ensure that we don't return an issue which is a child of the current issue.
 */
GH.RapidBoard.RapidListDataController.getNextIssueAfterSendToBottom = function(issueKey) {
    // get the id of the next issue
    var nextIssueKey = GH.RapidBoard.RapidListDataController.getNextIssueKey(issueKey);
    if (! nextIssueKey) return false;

    // get the position of the current issue and the next issue
    var currentPosition = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
    var nextPosition = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(nextIssueKey);

    // if the current issue is a sub-task, return the next issue id regardless
    if (currentPosition.issue.parentKey) {
        return nextIssueKey;
    }

    // search for the next issue which meets these criteria:
    //   is NOT a sub-task; OR
    //   the parent is NOT the current issue
    while (nextPosition.issue.parentKey && nextPosition.issue.parentKey == issueKey) {
        nextIssueKey = GH.RapidBoard.RapidListDataController.getNextIssueKey(nextIssueKey);
        if (!nextIssueKey) {
            // if we can't find any more issues, stop here
            return false;
        }
        nextPosition = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(nextIssueKey);
    }

    return nextIssueKey;
};

/**
 * Get last issue key for a given column
 */
GH.RapidBoard.RapidListDataController.getLastIssueKeyInColumn = function(columnIndex) {
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    for (var i=swimlanes.length-1; 0 <= i; i--) {
    	var issues = swimlanes[i].columns[columnIndex].issues;
        for (var j=issues.length-1;0 <= 0; j--) {
            if (! issues[j].isFiltered) return issues[j].key;
        }
	}
    return false;
};


/**
 * Get the id of the following issue.
 */
GH.RapidBoard.RapidListDataController.getNextIssueKey = function(issueKey) {
	var data = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	if (!data) return false;

    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();

	// try selecting the next in the same swimlane/column
	if (!hasSwimlanes || !GH.RapidBoard.State.isSwimlaneCollapsed(data.swimlane.id)) {
		if (data.issueVisibilityIndex + 1 < data.column.visibleIssueCount) {
	        for(var i = data.issueIndex + 1; i < data.column.issues.length; i++) {
	            if(!data.column.issues[i].isFiltered) {
	                return data.column.issues[i].key;
	            }
	        }
		}
	}

	// go through the rest of the swimlanes
    if(hasSwimlanes) {
        var columnIndex = data.columnIndex;
        var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
        for (var i = data.swimlaneIndex + 1; i < swimlanes.length; i++) {
            var swimlane = swimlanes[i];
            if (swimlane.visibleIssueCount < 1) continue;
            if (GH.RapidBoard.State.isSwimlaneCollapsed(swimlane.id)) continue;
            var column = swimlane.columns[columnIndex];
            if (column.visibleIssueCount < 1) continue;
            //Find the first visible issue in the next column
            for (var j = 0; j < column.issues.length; j++) {
                if(!column.issues[j].isFiltered) {
                    return column.issues[j].key;
                }
            }
        }
    }
	// not found
	return false;
};

/**
 * Get the id of the previous issue.
 */
GH.RapidBoard.RapidListDataController.getPreviousIssueKey = function(issueKey) {
	var data = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(issueKey);
	if (!data) return false;

    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();

	// try selecting the previous in the same swimlane/column
	if (!hasSwimlanes || !GH.RapidBoard.State.isSwimlaneCollapsed(data.swimlane.id)) {
		if (data.issueVisibilityIndex > 0) {
	        for(var i = data.issueIndex - 1; i >= 0; i--) {
	            if(!data.column.issues[i].isFiltered) {
	                return data.column.issues[i].key;
	            }
	        }
		}
	}

	// go through the rest of the swimlanes
    if(hasSwimlanes) {
        var columnIndex = data.columnIndex;
        var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
        for (var i = data.swimlaneIndex - 1; 0 <= i; i--) {
            var swimlane = swimlanes[i];
            if (swimlane.visibleIssueCount < 1) continue;
            if (GH.RapidBoard.State.isSwimlaneCollapsed(swimlane.id)) continue;
            var column = swimlane.columns[columnIndex];
            if (column.visibleIssueCount < 1) continue;
            //Find the last visible issue in the previous column
            for (var j = column.issues.length - 1; 0 <= j; j--) {
                if(!column.issues[j].isFiltered) {
                    return column.issues[j].key;
                }
            }
        }
    }
	// not found
	return false;
};

/**
 * Get the id for the visible issue in the given column at given visibility index.
 *
 * If the index does not exist then the last issue in the column is returned.
 * If no issue are available in the given column, false is returned
 */
GH.RapidBoard.RapidListDataController.getIssueKeyForColumnAtIndex = function(swimlaneIndex, columnIndex, issueIndex) {
	var swimlane = GH.RapidBoard.RapidListDataController.getSwimlane(swimlaneIndex);
	if (!swimlane) return false;
    if (swimlane.visibleIssueCount < 1) return false;
	var column = swimlane.columns[columnIndex];
	if (!column) return false;
	var issues = column.issues;
	var issueCount = issues.length;
	if (issueCount == 0) return false;
    var issueVisibilityCount = column.visibleIssueCount;
    if (issueVisibilityCount == 0) return false;
    // if issueIndex is higher than the number of visible issues then we want to return the last visible issue
    if (issueIndex >= issueVisibilityCount) {
        for (var i = issueCount - 1; i >= 0; i--) {
            if(!issues[i].isFiltered) {
                return issues[i].key;
            }
        }
    // otherwise we want to get the issueIndex-th visible issue.
    } else {
        var visibilityIndex = 0;
        for (var i = 0; i < issueCount; i++) {
            if(!issues[i].isFiltered) {
                if(visibilityIndex == issueIndex) {
                    return issues[i].key;
                }
                visibilityIndex++;
            }
        }
    }

	return false;
};

/**
 * Get the first issue in the view, starting with the first column
 */
GH.RapidBoard.RapidListDataController.getFirstIssueKey = function() {
	if (!GH.view.model.rapidList) return false;
	var cols = GH.RapidBoard.RapidListDataController.getAllColumns();
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();
    for (var colIndex=0; colIndex < cols.length; colIndex++) {
    	for (var h=0; h < swimlanes.length; h++) {
	    	var swimlane = swimlanes[h];
	    	if (hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(swimlane.id)) continue;
	    	if (swimlane.columns[colIndex].visibleIssueCount > 0) {
                for (var i=0; i < swimlane.columns[colIndex].issues.length; i++) {
                    if (!swimlane.columns[colIndex].issues[i].isFiltered) {
                        return swimlane.columns[colIndex].issues[i].key;
                    }
                }

	    	}
    	}
	}
    return false;
};

/**
 * Get the first visible issue in the specified issue's swimlane-column.
 * If the specified issue is a sub-task, it will retrieve the first visible sub-task in it's group.
 */
GH.RapidBoard.RapidListDataController.getFirstRankableIssueKeyInColumn = function(sourceIssueKey) {
	if (!GH.view.model.rapidList) return false;

    // retrieve position information about the source issue
    var position = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(sourceIssueKey);
    var column = position.column;

    for (var i = 0; i < column.issueCount; i++) {
        var issue = column.issues[i];
        if (!issue.isFiltered) {
            if (GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(issue.key, sourceIssueKey)) {
                return issue.key;
            }
        }
    }

    return false;
};

/**
 * Get the last visible issue in the specified issue's swimlane-column.
 * If the specified issue is a sub-task, it will retrieve the last visible sub-task in it's group.
 */
GH.RapidBoard.RapidListDataController.getLastRankableIssueKeyInColumn = function(sourceIssueKey) {
	if (!GH.view.model.rapidList) return false;

    // retrieve position information about the source issue
    var position = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(sourceIssueKey);
    var column = position.column;

    for (var i = column.issueCount - 1; i >= 0; i--) {
        var issue = column.issues[i];
        if (!issue.isFiltered) {
            if (GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(issue.key, sourceIssueKey)) {
                return issue.key;
            }
        }
    }

    return false;
};

/**
 * Get the last issue in the view (beginning in the first column!)
 */
GH.RapidBoard.RapidListDataController.getLastIssueKey = function() {
	if (!GH.view.model.rapidList) return false;
	var cols = GH.RapidBoard.RapidListDataController.getAllColumns();
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllSwimlanes();
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();
    for (var colIndex=0; colIndex < cols.length; colIndex++) {
    	for (var h=swimlanes.length-1; h >= 0; h--) {
	    	var swimlane = swimlanes[h];
	    	if (hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(swimlane.id)) continue;
	    	if (swimlane.columns[colIndex].visibleIssueCount > 0) {
                for (var i= swimlane.columns[colIndex].issues.length - 1; i >= 0; i--) {
                    if(!swimlane.columns[colIndex].issues[i].isFiltered) {
                        return swimlane.columns[colIndex].issues[i].key;
                    }
                }

	    	}
    	}
	}
    return false;
};

/**
` * Get the issue data for a given issue key
 */
GH.RapidBoard.RapidListDataController.getIssueDataByKey = function(issueKey) {
	var resultData = false;
	var callback = function(data) {
		if (data.key == issueKey) {
			resultData = data;
		}
	};
	GH.RapidBoard.RapidListDataController.applyToIssues(callback);
	return resultData;
};

/**
 * Tell if this rapid view supports ranking
 */
GH.RapidBoard.RapidListDataController.isRankable = function() {
    var data = GH.RapidBoard.RapidListDataController.getRapidListData();
    return data && data.rankable; 
};

/**
 * Are the two passed issues be ranked at the same time.
 */
GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether = function(keyOne, keyTwo) {
    // fetch data for both issues
    var dataOne = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(keyOne);
    var dataTwo = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(keyTwo);

    // ensure both are in the same swimlane
    if(dataOne.swimlane.id != dataTwo.swimlane.id) return false;

    // ensure both are in the same column
    if(dataOne.column.id != dataTwo.column.id) return false;

    // if both are top level issues they can be ranked together
    if (!dataOne.issue.parentKey && !dataTwo.issue.parentKey) return true;

    // if both are child issues of the same parent they can be ranked together
    if (dataOne.issue.parentKey == dataTwo.issue.parentKey) return true;

    // not rankable together (different parent or parent and child issue)
    return false;
};

/**
 * Get all issues between two issues (including the limits, in order fromKey to toKey).
 * If the two issues are incompatible to each other an empty array is returned.
 */
GH.RapidBoard.RapidListDataController.getRankableIssuesRange = function(fromKey, toKey) {
    // return an empty array if the two issue keys are incompatible
    if (!GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(fromKey, toKey)) return [];

	// ensure the range is valid, this also ensures we are in a single column of a single swimlane
	if (fromKey == toKey) return [fromKey];
	
    // fetch data for both issues
    var dataOne = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(fromKey);
    var dataTwo = GH.RapidBoard.RapidListDataController._getIssuePositionAndDataByKey(toKey);

    // initialize result array and add the first key
    var issues = [];
    issues.push(fromKey);

    var column = dataOne.column;
    var fromIndex = dataOne.issueIndex;
    var toIndex = dataTwo.issueIndex;
    if (toIndex > fromIndex) {
        // check top to bottom, skip fromKey
    	for (var i = fromIndex+1; i <= toIndex; i++) {
    		var issue = column.issues[i]; 
    		if (!issue.isFiltered && GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(fromKey, issue.key)) {
    			issues.push(issue.key);
    		}
    	}
    } else {
        // check bottom to to, skip fromKey
    	for (var i = fromIndex-1; i >= toIndex; i--) {
    		var issue = column.issues[i]; 
    		if (!issue.isFiltered && GH.RapidBoard.RapidListDataController.canIssuesBeRankedTogether(fromKey, issue.key)) {
    			issues.push(issue.key);
    		}
    	}
    }
    
    // return the issues
    return issues;
};


/**
 * Returns the issue to the left of the issue represented in issueData, increasing offset to savedIndex if higher than
 * issueData's. returns undefined when no such issue exists
 * @param issueData structure of the form returned by getIssueDataForKey
 */
GH.RapidBoard.RapidListDataController.getIssueKeyInPreviousColumn = function(issueData, savedIndex) {
	// abort if the current issue is hidden
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();
	if (hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(issueData.swimlane.id)) return;

	// continue searching
	var swimlaneIndex = issueData.swimlaneIndex;
	var currentColumn = issueData.columnIndex - 1;
	while(0 <= currentColumn) {
		var previousIssueKey = GH.RapidBoard.RapidListDataController.getIssueKeyForColumnAtIndex(swimlaneIndex, currentColumn, savedIndex);
		if (previousIssueKey) {
			return previousIssueKey;
		}
		currentColumn--;
	}
    return;
};
/**
 * Returns the issue to the right of the issue represented in issueData, increasing offset to savedIndex if higher than
 * issueData's. returns undefined when no such issue exists
 * @param issueData structure of the form returned by getIssueDataForKey
 */
GH.RapidBoard.RapidListDataController.getIssueKeyInNextColumn = function(issueData, savedIndex) {
	// abort if the current issue is hidden
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();
	if (hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(issueData.swimlane.id)) return;

	// continue searching
	var columnCount = GH.RapidBoard.RapidListDataController.getColumnCount();
	var swimlaneIndex = issueData.swimlaneIndex;
	var currentColumn = issueData.columnIndex + 1;
	while(currentColumn < columnCount) {
		var nextIssueKey = GH.RapidBoard.RapidListDataController.getIssueKeyForColumnAtIndex(swimlaneIndex, currentColumn, savedIndex);
		if (nextIssueKey) {
			return nextIssueKey;
		}
		currentColumn++;
	}
};

GH.RapidBoard.RapidListDataController.issueIsInCollapsedSwimlaneByKey = function (issueKey) {
    var swimlaneId = GH.RapidBoard.RapidListDataController.getSwimlane(GH.RapidBoard.RapidListDataController.getSwimlaneIndexByKey(issueKey)).id;
    var hasSwimlanes = GH.RapidBoard.RapidListDataController.hasSwimlanes();
    return hasSwimlanes && GH.RapidBoard.State.isSwimlaneCollapsed(swimlaneId);
};

/**
 * For a given location, find the issue at, or nearest to it (dropping to the top of the following
 * swimlane if we've overshot the specified swimlane/column combo).
 * @location the specified location, of the form {swimlaneId, columnId, position}
 */
GH.RapidBoard.RapidListDataController.getIssueDataAtLocation = function (location) {

    // if location doesn't have the shape we expect return undefined
    if(location == undefined
            || location.swimlaneId == undefined
            || location.columnId == undefined
            || location.position == undefined ) {
        return undefined;
    }
    var selectedSwimlane, selectedColumn, selectedIssue, nextSwimlane, visIndex = 0;

    // select the swimlane with location.swimlaneId
    var swimlanes = GH.RapidBoard.RapidListDataController.getAllVisibleSwimlanes();
    for(var i = 0, l = swimlanes.length; i < l; i++) {
        if(swimlanes[i].id == location.swimlaneId) {
            selectedSwimlane = swimlanes[i];
            if (i+1 < l) {
                nextSwimlane = swimlanes[i+1];
            }
            break;
        }
    }
    if(!selectedSwimlane) return undefined;

    // select the column in selectedSwimlane with location.columnId
    for(i = 0, l = selectedSwimlane.columns.length; i < l; i++) {
        if(selectedSwimlane.columns[i].id == location.columnId) {
            selectedColumn = selectedSwimlane.columns[i];
            break;
        }
    }
    if(!selectedColumn ) return undefined;

    // select the issue from the selected column
    for(i = 0, l = selectedColumn.issues.length; i < l; i++) {
        if(!selectedColumn.issues[i].isFiltered) {
            if(visIndex == location.position) {
                selectedIssue = selectedColumn.issues[i];
                break;
            } else {
                visIndex++;
            }
        }
    }

    // if we got an issue, return it
    if (selectedIssue) {
        return selectedIssue;
    } else if (nextSwimlane) {
        // if we didn't get an issue, then we want to grab the top issue from the next swimlane, so recurse
        return GH.RapidBoard.RapidListDataController.getIssueDataAtLocation({
            swimlaneId: nextSwimlane.id,
            columnId: location.columnId,
            position: 0
        });
    } else {
        return GH.RapidBoard.RapidListDataController.getIssueDataAtLocation({
            swimlaneId: location.swimlaneId,
            columnId: location.columnId,
            position: visIndex - 1
        });
    }
};