/**
 * Rapid view selector component
 */

GH.RapidBoard.RapidViewSelector = {};

/**
 * Get the available rapid views.
 */
GH.RapidBoard.RapidViewSelector.getRapidViews = function() {
    return GH.view.model.rapidViews && GH.view.model.rapidViews.success ? GH.view.model.rapidViews.success.rapidViews : [];
};

/**
 * Get the selected rapid view.
 */
GH.RapidBoard.RapidViewSelector.getSelectedView = function() {
    return GH.view.model.selectedView && GH.view.model.selectedView.success ? GH.view.model.selectedView.success : undefined;
};

/**
 * Renders the component.
 */
GH.RapidBoard.RapidViewSelector.renderComponent = function() {
    // check whether loading of the rapidViews failed
    if (!GH.view.model.rapidViews.success) {
        GH.RapidBoard.PageLoadErrorView.showPageLoadErrorMessage(GH.view.model.rapidViews.error);
        return;
    }

    // fetch the views, from now on we have an array of views
    var rapidViews = GH.RapidBoard.RapidViewSelector.getRapidViews();
    if (rapidViews.length < 1) {
        // show welcome message only
        GH.RapidBoard.welcome();
        return;
    }

    // render the header html
    var selectedView = GH.RapidBoard.RapidViewSelector.getSelectedView();
    var hasSelectedView = selectedView ? true : false;
    var selectedViewName = hasSelectedView ? selectedView.name : AJS.I18n.getText('gh.rapid.board.error.noview');
    var hasDropdown = hasSelectedView ? rapidViews.length > 1 : rapidViews.length > 0;

    // render the header
    AJS.$('#ghx-header').html(GH.tpl.rapid.navigation.renderHeader({name: selectedViewName, hasDropdown: hasDropdown}));

    // render the heroes
    if(hasSelectedView && selectedView.canEdit){
        AJS.$('#ghx-header').append(GH.tpl.rapid.navigation.renderHero({id: selectedView.id}));
    }

    // update the page title
    GH.RapidBoard.RapidViewSelector.updatePageTitle(selectedView);

    // render the selector html
    GH.RapidBoard.RapidViewSelector.renderDropdown(rapidViews, selectedView);

    // only render the operations and the query in case we got a selected view
    if (hasSelectedView) {
        GH.RapidBoard.PageLoadErrorView.clear();
    } else {
        // show errors
        GH.RapidBoard.PageLoadErrorView.showPageLoadErrorMessage(GH.view.model.selectedView.error.errors);
    }
};


/**
 * Renders a Welcome Screen
 */
GH.RapidBoard.welcome = function() {

    // render the header html
    AJS.$('#ghx-header').html(GH.tpl.rapid.navigation.renderSimpleHeader({name : AJS.I18n.getText('gh.rapid.name')}));

    // No Views template
    AJS.$('#ghx-pool').html(GH.tpl.board.x.renderNoViews());

};

/**
 * Holds the original page title.
 * We set this value on selector init
 */
GH.RapidBoard.RapidViewSelector.origPageTitle = undefined;

/**
 * Updates the page title with the selected rapid view name
 */
GH.RapidBoard.RapidViewSelector.updatePageTitle = function(selectedView) {
    if (selectedView) {
        document.title = selectedView.name + ' - ' + GH.RapidBoard.RapidViewSelector.origPageTitle;
    } else {
        document.title = GH.RapidBoard.RapidViewSelector.origPageTitle;
    }
};

GH.RapidBoard.RapidViewSelector.viewModelComparator = function(a, b) {
    //ECMA standards, return -1, 0, or 1 (other browsers still accept true/false but not chrome)
    if(typeof a.name !== "string") {
        return -1;
    } else if (typeof b.name !== "string") {
        return 1;
    } else {
        return (a.name.toLowerCase() < b.name.toLowerCase()) ? -1 : (a.name.toLowerCase() > b.name.toLowerCase()) ? 1 : 0;
    }
};

/**
 * Renders the dropdown menu for the rapid views
 */
GH.RapidBoard.RapidViewSelector.renderDropdown = function(rapidViews, selectedView) {
    // don't proceed if we got less than 2 rapid views, leave as label
//	if (rapidViews.length < 2) return;

    // do a case insensitive sort of rapidViews by their name
    rapidViews.sort(GH.RapidBoard.RapidViewSelector.viewModelComparator);

    // render the dropdown choices
    AJS.$('#ghx-views').html(GH.tpl.rapid.navigation.renderOpsFilters({rapidViews: rapidViews, selectedViewId: selectedView ? selectedView.id : ''}));

    // fixes drop down issue when rapid board runs inside an iframe
    //AJS.params.ignoreFrame = true;
    
    // create the dropdown
    var dropdowns = AJS.Dropdown.create({
                trigger: AJS.$("#ghx-header .ghx-views.aui-dd-link"),
                content: AJS.$("#ghx-views .aui-list"),
                alignment: AJS.LEFT  	
            });

    
    // bind a change event to the select
    //AJS.$(window.parent.document).delegate('#ghx-views-select .aui-list-item', 'click', function(event) {
    AJS.$('#ghx-views-select .aui-list-item').bind('click', function() {

        // check the filter to load, return if we already got the right filter
        var rapidViewId = AJS.$(this).attr('data-value');
        rapidViewId =  parseInt(rapidViewId);

        // hide the dropdown
        AJS.$(dropdowns).each(function(index, dropdown) {
            dropdown.hide();
        });
        // hide the WelcomeMat success message, if present
        var welcomeMessage = AJS.$('#ghx-welcome-message');
        if (welcomeMessage.length > 0) {
            AJS.$('#ghx-welcome-message div.aui-message').closeMessage();
        }

        // set the state, then trigger loading of the data
        GH.RapidBoard.showRapidView(rapidViewId);

    });
};

/**
 * Toggles reporting on and off
 */
GH.RapidBoard.RapidViewSelector.toggleReporting = function() {
    if (!GH.RapidBoard.State.isChartViewOpened()) {
        if(!GH.RapidBoard.UserData.hasUserSeenChartIntro) {
            // if the user hasn't seen the intro before then show it
            GH.ChartIntro.openIntro();
        }
        // and now open the chart view, either showing the intro
        GH.RapidBoard.ChartView.openChartView();
    }
};

/**
 * Updates the reporting button ui according to whether a chart is displayed or not.
 */
GH.RapidBoard.RapidViewSelector.updateReportingButton = function() {
    // set/remove the active state on the button
    var buttonReporting = AJS.$('#reporting-toggle');
    var buttonWorking = AJS.$('#working-toggle');
    if (GH.RapidBoard.State.isChartViewOpened()) {
        buttonReporting.removeClass('active');
        buttonWorking.addClass('active');
    } else {
        buttonReporting.addClass('active');
        buttonWorking.removeClass('active');
    }
};

/**
 * Reporting
 */
GH.RapidBoard.RapidViewSelector.toggleWorking = function() {
    if (GH.RapidBoard.State.isChartViewOpened()) {
        GH.RapidBoard.ChartView.closeChartView();
        // if the user has seen the intro then unset intro as we leave the reporting screen
        if(GH.RapidBoard.UserData.hasUserSeenChartIntro) {
            GH.RapidBoard.State.setChartIntro(false);
        }
    }
};

/**
 * Display the rapid view configuration page
 */
GH.RapidBoard.RapidViewSelector.gotoRapidViewConfiguration = function(rapidViewId) {
    window.location.href = GH.Ajax.CONTEXT_PATH + '/secure/RapidView.jspa?rapidView=' + rapidViewId;
};

/**
 * Component initialization.
 */
GH.RapidBoard.RapidViewSelector.init = function() {
    // update all values when the model changes.
    AJS.$(document).bind('modelChanged', GH.RapidBoard.RapidViewSelector.renderComponent);

    AJS.$(document).bind('showChartView hideChartView', GH.RapidBoard.RapidViewSelector.updateReportingButton);

    // If there were page load errors, we still want to render this section to
    // allow the user to select a non-faulty rapid view.
    //AJS.$(document).bind('pageLoadErrors', GH.RapidBoard.RapidViewSelector.renderErrorComponent);

    // create RapidView button
    AJS.$(document).delegate('#ghx-rapidview-add', 'click', function(event) {
        GH.RapidBoard.CreateDialog.showCreateDialog();
        event.preventDefault();
    });

    // Register the link handler to redirect to the manage view page
    AJS.$(document).delegate('#ghx-rapidview-manage', 'click', function(event) {
        event.preventDefault();

        // fetch the selected view id from the state
        var rapidViewId = GH.RapidBoard.State.getRapidViewId();
        if (rapidViewId) {
            GH.RapidBoard.RapidViewSelector.gotoRapidViewConfiguration(rapidViewId);
        }

    });

    // listener for reporting toggle button
    AJS.$(document).delegate('#reporting-toggle', 'click', GH.RapidBoard.RapidViewSelector.toggleReporting);
    AJS.$(document).delegate('#working-toggle', 'click', GH.RapidBoard.RapidViewSelector.toggleWorking);

    // retain the document title, as we adapt it on view selection
    GH.RapidBoard.RapidViewSelector.origPageTitle = document.title;
};
