/**
 * Keeps the column headers and swimlane label in place when scrolling further down the page.
 * 
 * When modifying this, take the following situations into account:
 * 
 * - stalker bar has to resize on window width change
 * - swimlane label has to change on scroll
 * - stalker has to reposition when changing in and out of projector mode (Z-key)
 * - swimlane label should not show if there is only one lane
 * - column statistics change the position
 * - swimlanes can be collapsed
 */
GH.RapidBoard.Stalker = {};

/**
 * Holds the value for the column head offset
 */
GH.RapidBoard.Stalker.element = null;
GH.RapidBoard.Stalker.colHeadOffset = null;
GH.RapidBoard.Stalker.colHeadProjectorOffset = null;

/**
 * Compute the Stalker Column Header.
 */
GH.RapidBoard.Stalker.colHeadStalker = function() {
    // we might not have this around always. No need for stalker if there's no column header.
    var headerGroup = AJS.$('#ghx-column-header-group');
    if (headerGroup.length == 0) return;

    // fetch the header offset as well as the window top
    var stalkerHeader = GH.RapidBoard.Stalker.element;
    var headOffset = GH.RapidBoard.Stalker.getHeadOffset(headerGroup);
    var wScroll = AJS.$(window).scrollTop();

    // if scrolled further than offset, make it sticky
    if (wScroll > (headOffset-3)){
    	// ensure we got a stalker element
    	if (! stalkerHeader) {
    		// create a copy of the header group, which will be always positioned as fixed
    		stalkerHeader = headerGroup.clone().addClass('ghx-fixed');
    		headerGroup.after(stalkerHeader); // keep after, as code fetches element by id and once we got a stalker we have two in the document. TODO: replace css by classes
    		GH.RapidBoard.Stalker.element = stalkerHeader;
    	}
    	
        // position the stalker bar
        GH.RapidBoard.Stalker.position(stalkerHeader);
        
        // set the correct stalker label
        GH.RapidBoard.Stalker.setCurrentLabel(stalkerHeader, wScroll);
        
        // show the stalker
        stalkerHeader.show();
    } else {
    	// remove the stalker
    	if (stalkerHeader) {
    		stalkerHeader.remove();
    		GH.RapidBoard.Stalker.element = null;
    	}
    }
};

/**
 * Get the offset of the header from the top of the page. This is different for projector mode, but
 * we cache both values since they don't change and are expensive to calculate.
 */
GH.RapidBoard.Stalker.getHeadOffset = function(headerGroup) {
    var elOffset = GH.RapidBoard.SwimlaneView.colHeadOffset;
    var elProjectorOffset = GH.RapidBoard.SwimlaneView.colHeadProjectorOffset;
    
    // Check to see if Projector Mode enabled since offset values are different
    if (GH.Projector.isEnabled()){
        // check to see if column header offset is stored else measure it, then store it
        if (elProjectorOffset == null){
            elProjectorOffset = AJS.$(headerGroup).offset().top;
            GH.RapidBoard.SwimlaneView.colHeadProjectorOffset = elProjectorOffset;
        }
        return elProjectorOffset;
        
    } else {
        // check to see if column header offset is stored else measure it, then store it
        if (elOffset == null){
            elOffset = AJS.$(headerGroup).offset().top;
            GH.RapidBoard.SwimlaneView.colHeadOffset = elOffset;
        }
        return elOffset;
    }
};

/**
 * Fix the stalker position, aligned to the pool
 * 
 * @param el : The stalking column header
 */
GH.RapidBoard.Stalker.position = function(headerGroup) {
    var refEl = AJS.$('#ghx-pool');
    var elPos = refEl.offset().left;
    var elWidth = refEl.width();
    //headerGroup.addClass('ghx-fixed');
    headerGroup.css({'left':elPos, 'width':elWidth});
}; 

/**
 * Find the right swimlane label to stick under the stalker. This should be the lane we're currently looking at.
 * 
 * @returns true if a swimlane label was found, false otherwise.
 */
GH.RapidBoard.Stalker.setCurrentLabel = function(stalkerHeader, wScroll) {
	// update the swimlane name
    var labelHolder = stalkerHeader.find('#ghx-swimname');    
    var labelFound = false;

    // we don't display sticky lane labels in IE8, since it's not capable of computing offset efficiently.
    if (! (AJS.$.browser.msie && AJS.$.browser.version.indexOf("8") === 0)) {
        // set current swimlane label on stalker
        labelFound = GH.RapidBoard.Stalker.findAndUpdateLabel(stalkerHeader, wScroll);
    }

    // show the current swimlane label sticking to the stalker
    if(labelFound) {
        labelHolder.show();
        stalkerHeader.addClass('ghx-fixed-labeled');
    } else {
        labelHolder.hide();
        stalkerHeader.removeClass('ghx-fixed-labeled');
    }
};

/**
 * Find the right swimlane label to stick under the stalker. This should be the lane we're currently looking at.
 * 
 * @returns true if a swimlane label was found, false otherwise.
 */
GH.RapidBoard.Stalker.findAndUpdateLabel = function(stalkerHeader, wScroll) {
    var elHeight = AJS.$(stalkerHeader).height();    
    var currentLane = null;
    
    // function to calculate the swimlane header offset
    var calcSwimlaneOffset = function(swimlaneHeader) {
    	return swimlaneHeader.offset().top + ( swimlaneHeader.height() / 2 ) - elHeight;
    };
    
    // find the latest label - the first is always taken if available
    // we basically accept each lane, and fail when we encounter the first lane that is below the stalker
    AJS.$('.ghx-swimlane').each(function(index, elem) {
        // only if the lane has a header
        var swimlaneHeader = AJS.$(elem).find('.ghx-swimlane-header');
        if(swimlaneHeader.length > 0) {
        	// always accept the first
        	if (index == 0) {
        		currentLane = this;
        	}
            // calculation based on swimlane header label position + height + half its height for the transition to occur when on the stalk line
        	else if (calcSwimlaneOffset(swimlaneHeader) < wScroll) {
                currentLane = this;
            } else {
                return false;
            }
        }
    });
    
    // if we found a matching lane label, set the values on the label holder, and mark the label for display
    var labelFound = false;
    if (currentLane) {
        var laneHeader = AJS.$(currentLane).children('.ghx-swimlane-header');
        if (laneHeader.length > 0) {
        	var swimname = stalkerHeader.find('#ghx-swimname');
        	
        	// set the name
        	swimname.find('span').text(laneHeader.attr('data-swimname'));
        	
        	// adapt the bottom of the element 
        	var topDistance = calcSwimlaneOffset(laneHeader);
        	var bottom = wScroll - topDistance + 3; // pure magic! no idea where these 3 pixels come from!!!
        	// we fix the position at 4 
        	if (bottom > 4) {
        		bottom = 4;
        	}
        	swimname.css({'bottom': bottom});
        	
        	// label found, display
        	return true;
        }
    }
    
    return false;
};



/**
 * Bind the events to scroll/resize. We don't use a delay here since it would make the UI choppy.
 */
GH.RapidBoard.Stalker.init = function() {
    // if the page is already scrolled (reload from the middle), we'll receive scroll events so the stalker will be positioned.
    AJS.$(window).bind('scroll', GH.RapidBoard.Stalker.colHeadStalker);
    
    // ok, now this is a little weird. The rather idiotic flot resize plugin fires off window resize events on page load,
    // presumably to initialise itself somehow. The thing is, we DON'T want the stalker to fire if the window is not scrolled,
    // so we bind the resize with a little delay to avoid any trouble with flot or anyone else firing early resize events.
    window.setTimeout(function() {
        AJS.$(window).bind('resize', GH.RapidBoard.Stalker.colHeadStalker);
    }, 500);    // magic number by empirical measurement
};
