/**
 * Swimlane view controller.
 * 
 * Note: this file is the view part of the RapidListDataContoller
 */
GH.RapidBoard.SwimlaneView = {};

/**
 * Holds the columns displayed by the swimlane
 */
GH.RapidBoard.SwimlaneView.displayedColumns = [];

/**
 * Is the column with the given id displayed?
 */
GH.RapidBoard.SwimlaneView.isColumnDisplayed = function(columnId) {
	for (var i=0; i < GH.RapidBoard.SwimlaneView.displayedColumns.length; i++) {
		if (GH.RapidBoard.SwimlaneView.displayedColumns[i].id == columnId) {
			return true;
		}
	}
	return false;
};

/**
 * Renders the swim lanes into the pool
 */
GH.RapidBoard.SwimlaneView.renderSwimlanes = function() {
	// empty the content
	var container = AJS.$('#ghx-pool');
	//container.empty();
	//container.hide().empty().show();

	// show errors if we got some
	var errors = GH.RapidBoard.RapidListDataController.getRapidListErrors();
	if (errors) {
		//AJS.$('#ghx-pool').html(GH.tpl.rapid.pageLoadError.renderErrors({errors:errors}));
		container[0].innerHTML = GH.tpl.rapid.pageLoadError.renderErrors({errors:errors});
		return;
	}
	
	// sanity check
	var columns = GH.RapidBoard.RapidListDataController.getAllColumns();
	if (columns.length < 1) {
		container[0].innerHTML = '';
		return;
	}
	
	// Reduce columns to 1 in case the details view is displayed
	var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey(true);
	if (GH.RapidBoard.State.isDetailsViewOpened()) {
		// find the issue and its column, then display the details view
		var columnIndex = GH.RapidBoard.RapidListDataController.getColumnIndexByKey(issueKey);
		if (columnIndex) {
			columns = [ columns[columnIndex ]];
		} else {
			// fetch the first column that got issues
			var index = 0;
			for (var i=0; i < columns.length; i++) {
				if (columns[i].issueCount > 0) {
					index = i;
					break;
				}
			}
			columns = [ columns[index] ];
		}
	}

	// set the displayed columns
	GH.RapidBoard.SwimlaneView.displayedColumns = columns;

	// render each swimlane
	var statistics = GH.RapidBoard.RapidListDataController.getStatistics();
    //var headerDiv = AJS.$(GH.tpl.rapid.swimlane.renderColumnsHeader({ columns:columns, statistics:statistics }));
    //container.append(headerDiv);
	var html = GH.tpl.rapid.swimlane.renderColumnsHeader({ columns:columns, statistics:statistics });

    // first filter the swimlanes to get the visual ones
    var visualSwimlanes = GH.RapidBoard.RapidListDataController.getAllVisibleSwimlanes();

	// now render all swimlanes, and keep them in an array
	for (var i=0; i < visualSwimlanes.length; i++) {
		var swimlane = visualSwimlanes[i];
		var swimlaneHtml = GH.tpl.rapid.swimlane.renderSwimlane({
            columns:columns,
            swimlane:swimlane,
            visualIndex: i,
            collapsed: swimlane.collapsed
        });
		//var swimlaneDiv = AJS.$(swimlaneHtml);
		//container.append(swimlaneDiv);
		html += swimlaneHtml;
	}
	
	// set the new content
	container[0].innerHTML = html;

	// update the banding class, as the number of columns has changed
	GH.RapidBoard.SwimlaneView.updateBandingClass();
	                             swimlane
	// set constraint busted class on container
	GH.RapidBoard.SwimlaneView.setBusted(container);
	
	// add a class that defines whether more than one swimlane is displayed. Required to show/hide the swimlane name
	if (visualSwimlanes.hasSwimlanes) { // Adds the class for 1+ non-default swimlane
		container.addClass('ghx-has-swimlanes');
	} else {
		container.removeClass('ghx-has-swimlanes');
	}
	
	// update the stalker
	GH.RapidBoard.Stalker.colHeadStalker();
	
	// ensure the issue is visible
	if (issueKey) {
		GH.RapidBoard.Issue.gotoIssue(issueKey);
	}
	
	// swimlane got rendered, inform interested parties
	AJS.$(document).trigger('swimlaneRendered');

	return container;
};

/**
 * Updates the column quantities class set accordingly to the currently displayed column count.
 */
GH.RapidBoard.SwimlaneView.setBusted = function(targetElement) {
    // Denotes at least one column is busted
    var rapidListData = GH.RapidBoard.RapidListDataController.getRapidListData();
    if (rapidListData && rapidListData.anyColumnBusted) {
        AJS.$(targetElement).addClass('ghx-busted');
    } else {
        AJS.$(targetElement).removeClass('ghx-busted');
    }
};


/**
 * Checks whether the current selected issue is visible, ensures the correct column is displayed if so
 */
GH.RapidBoard.SwimlaneView.ensureSelectedIssueColumnVisible = function() {
	if (!GH.RapidBoard.ViewController.isPoolDisplayed()) return;
	if (!GH.RapidBoard.State.isDetailsViewOpened()) return;
	
	// check whether we have to re-render
	var issueKey = GH.RapidBoard.Issue.getSelectedIssueKey();
	var columnIndex = GH.RapidBoard.RapidListDataController.getColumnIndexByKey(issueKey);
	if (!GH.RapidBoard.SwimlaneView.isColumnDisplayed(columnIndex)) {
		GH.RapidBoard.SwimlaneView.renderSwimlanes();
	}
};

/**
 * Called upon page load. Registers events this component is interested in
 */
GH.RapidBoard.SwimlaneView.init = function() {
	AJS.$(document).bind('updateSwimlaneView', GH.RapidBoard.SwimlaneView.renderSwimlanes);
	AJS.$(document).bind('issueSelected', GH.RapidBoard.SwimlaneView.ensureSelectedIssueColumnVisible);

    // swimlane toggles
    AJS.$('#ghx-pool').delegate('.ghx-swimlane-header h2 span', 'click', GH.RapidBoard.SwimlaneView.toggleCollapsedHandler);
    AJS.$('#ghx-operations').delegate('#js-swimlanes-close', 'click', GH.RapidBoard.SwimlaneView.collapseAll);
    AJS.$('#ghx-operations').delegate('#js-swimlanes-open', 'click', GH.RapidBoard.SwimlaneView.expandAll);
    
    // init resize handler for the banding class
    GH.RapidBoard.SwimlaneView.initBandingResizeListener();
};

/**
 * Initializes a resize listener that kicks off the updating of the banding class on the pool. 
 */
GH.RapidBoard.SwimlaneView.initBandingResizeListener = function() {
	var resizeDelay;
    AJS.$(window).bind('resize', function() {
        window.clearTimeout(resizeDelay);
        resizeDelay = window.setTimeout(GH.RapidBoard.SwimlaneView.updateBandingClass, 250);
    });
};

/**
 * Responsive Issues Classes - used to change stuff depending on available space
 */
GH.RapidBoard.SwimlaneView.updateBandingClass = function() {
	// the banding class depends on the number of columns displayed
	var columnCount = GH.RapidBoard.SwimlaneView.displayedColumns.length;
	
    var colWidth = (AJS.$(window).width() - 34) / columnCount; // magic number is 16padding, 16margin, 2border @todo better way without reflow?

    // remove old values
    var container = AJS.$('#ghx-pool'); 
    container.attr("className", container.attr("className").replace(/ghx-band-\d+/g, ""));

    // class depending on issue widths
    if (colWidth <= 185) {
        container.addClass('ghx-band-1');
    }
    else if (colWidth > 185 && colWidth <= 220) {
        container.addClass('ghx-band-2');
    }
};

/**
 * Collapse/Expand 1 Swimlane
 */
GH.RapidBoard.SwimlaneView.toggleCollapsedHandler = function() {
    var swimlane = AJS.$(this).closest('.ghx-swimlane');
    GH.RapidBoard.SwimlaneView.setCollapsed(swimlane, !swimlane.hasClass('ghx-closed'));
};

GH.RapidBoard.SwimlaneView.setCollapsed = function(swimlane, collapsed) {
    var swimlaneId = parseInt(swimlane.attr('swimlane-id'));
    // toggle and store change
    collapsed ? swimlane.addClass('ghx-closed') : swimlane.removeClass('ghx-closed');

    GH.RapidBoard.State.setSwimlaneCollapsed(swimlaneId, collapsed);
};

GH.RapidBoard.SwimlaneView.setCollapsedById = function(swimlaneId, collapsed) {
    AJS.$(".ghx-swimlane").each(function(){
        var $this = AJS.$(this);
        if($this.attr("swimlane-id") == swimlaneId) {
            if($this.hasClass('ghx-closed') != collapsed) {
                GH.RapidBoard.SwimlaneView.setCollapsed($this, collapsed);
            }
        }
    });
};

/**
 * Collapse All Swimlanes
 */
GH.RapidBoard.SwimlaneView.collapseAll = function() {
    AJS.$('.ghx-swimlane')
    	.addClass('ghx-closed')
		.each(function () {
			GH.RapidBoard.State.setSwimlaneCollapsed(parseInt(AJS.$(this).attr('swimlane-id')), true);
	    });
};

/**
 * Expand All Swimlanes
 */
GH.RapidBoard.SwimlaneView.expandAll = function() {
    AJS.$('.ghx-swimlane').removeClass('ghx-closed')
    	.each(function () {
    		GH.RapidBoard.State.setSwimlaneCollapsed(parseInt(AJS.$(this).attr('swimlane-id')), false);
	    });
};
