/**
 * View Controller
 * 
 * Keeps track of the current view stage and adapts the column accordingly.
 */

GH.RapidBoard.ViewController = {};

/**
 * Displays the details view for a given issue.
 */
GH.RapidBoard.ViewController.showDetailsView = function(columnIndex) {

    // Set min-height for content area
    var minHeight = AJS.$(window).height() - (AJS.$("#header").height() + AJS.$("div.footer").height() + AJS.$("#ghx-header").height());
    AJS.$('#ghx-content-main').addClass('ghx-has-detail').css({"min-height":minHeight});

	// show the details view
	GH.RapidBoard.DetailsView.getContainer().show();

	// update the fixed elements.
	GH.RapidBoard.ViewController.setFixedElements([ GH.RapidBoard.DetailsView.getFixedElement() ]);
	
	// update the details view
	AJS.$(document).trigger('updateDetailsView');
	AJS.$(document).trigger('updateSwimlaneView');
};

/**
 * Displays the column view.
 */
GH.RapidBoard.ViewController.hideDetailsView = function() {
	// hide the details view
	GH.RapidBoard.DetailsView.getContainer().hide();
	AJS.$('#ghx-content-main').removeClass('ghx-has-detail').css({"min-height":""});
	
	// update the fixed elements.
	GH.RapidBoard.ViewController.setFixedElements([]);
	
	AJS.$(document).trigger('updateSwimlaneView');
};

/**
 * Updates the view to reflect whatever should be selected
 */
GH.RapidBoard.ViewController.updatePool = function() {
	if (GH.RapidBoard.State.isDetailsViewOpened()) {
		GH.RapidBoard.ViewController.showDetailsView();
	} else {
		GH.RapidBoard.ViewController.hideDetailsView();
	}
};

/**
 * Shows the chart view
 */
GH.RapidBoard.ViewController.showChart = function(show) {
	AJS.$('#ghx-pool').hide();
	AJS.$('#ghx-detail').hide();
	AJS.$('#ghx-display').show();
    AJS.$(".ghx-timeframe").show();
	AJS.$(document).trigger('loadChart');
};

/**
 * Shows the pool
 */
GH.RapidBoard.ViewController.showPool = function(show) {
	AJS.$('#ghx-display').hide();
	AJS.$('#ghx-pool').show();
	GH.RapidBoard.ViewController.updatePool();
};

/**
 * Is the pool currently displayed
 * PERF visible pseudo selector is expensive, this should just be a boolean look-up into RapidBoardState.js
 * PERF Rewrite whole file
 */
GH.RapidBoard.ViewController.isPoolDisplayed = function() {
	return AJS.$('#ghx-pool:visible').length > 0;
};


/**
 * Lays out the UI
 */
GH.RapidBoard.ViewController.updateAll = function() {
	// don't show anything if we don't have a selected view
	if (! GH.RapidBoard.RapidViewSelector.getSelectedView()) {
		return;
	}
	if (GH.RapidBoard.State.isChartViewOpened()) {
		GH.RapidBoard.ViewController.showChart();
	} else {
		GH.RapidBoard.ViewController.showPool();
	}
};

/**
 * Initializes the view controller.
 */
GH.RapidBoard.ViewController.init = function() {
	AJS.$(document).bind('modelChanged showChartView hideChartView', GH.RapidBoard.ViewController.updateAll);
	AJS.$(document).bind('showDetailsView', GH.RapidBoard.ViewController.showDetailsView);
	AJS.$(document).bind('hideDetailsView', GH.RapidBoard.ViewController.hideDetailsView);
	AJS.$(document).bind('swimlaneRendered', GH.RapidBoard.ViewController.layoutFixedElements);
};

/**
 * Fixed elements support
 */
GH.RapidBoard.ViewController.fixedElements = [];

GH.RapidBoard.ViewController.setFixedElements = function(elements) {
	GH.RapidBoard.ViewController.fixedElements = elements;
	GH.RapidBoard.ViewController.initFixedElements();
};

/**
 * Initializes the fixed elements listener
 */
GH.RapidBoard.ViewController.initFixedElements = function() {
	// first unbind, then bind if necessary
	AJS.$(window).unbind('scroll resize', GH.RapidBoard.ViewController.layoutFixedElements);
	
	if (GH.RapidBoard.ViewController.fixedElements.length > 0) {
		// register handler
		AJS.$(window).bind('scroll resize', GH.RapidBoard.ViewController.layoutFixedElements);
		
		// initial layout
		GH.RapidBoard.ViewController.layoutFixedElements();
	}	
};

/**
 * Updates all columns marked as fixed to ensure they fit within the content view area
 */
GH.RapidBoard.ViewController.layoutFixedElements = function() {
    var fixedElements = GH.RapidBoard.ViewController.fixedElements; 

    // computing the content area dimensions is an expensive operation, only do if necessary
    if (fixedElements.length > 0) {
        // fetch the dimensions of the view content area
        var dimension = GH.RapidBoard.ViewController.getContentAreaDimension();
        
        for (var i=0; i<fixedElements.length; i++) {
            var fixedElement = fixedElements[i];
            var elem = fixedElement.element;

            elem.css({
                'top': dimension.top,
                'height': dimension.height -10 // @todo magic number
            });

            // invoke callback if provided
            if(fixedElement.resizeCallback){
                fixedElement.resizeCallback(elem);
            }
        }
    }
};

/**
 * Calculates the size of the visible column area. The returned values, top and height
 * are to be used with fixed position.
 */ 
GH.RapidBoard.ViewController.getContentAreaDimension = function() {
    
    // actual top of the column and window
    var contentTop = AJS.$('#ghx-pool').offset().top;
    var windowTop = AJS.$(window).scrollTop();
    var windowHeight = AJS.$(window).height();
    var contentMain = AJS.$('#ghx-content-main');
    var mainBottomInner = contentMain.offset().top + contentMain.outerHeight() - ((contentMain.outerHeight() - contentMain.innerHeight()) / 2);
    var dimension = {};
    
    // calculate top
    var topDistance = contentTop - windowTop; 
    dimension.top = topDistance > 0 ? topDistance : 0;
    
    // for the height we have to take two things into account
    dimension.height = windowHeight;
    // 1. reduce the height by top
    dimension.height -= dimension.top;
    // 2. reduce the height by possible visible footer
    var bottomDistance = (windowTop + windowHeight - mainBottomInner);
    dimension.height -=  bottomDistance > 0 ? bottomDistance : 0;

    return dimension;
};
