/**
 * Control Chart logic
 */
GH.RapidBoard.ChartView.ControlChart = {};

/**
 * Shows the chart for the provided data
 */
GH.RapidBoard.ChartView.ControlChart.showChart = function(data) {
	// first set the data
	GH.ControlChart.Data.setRawData(data);

    // hide the spinner
    GH.RapidBoard.ChartView.getChartStatus().hide();

    // render the data part
    GH.RapidBoard.ChartView.getChartGroup().show();
    
    // then render the chart
    GH.RapidBoard.ChartView.ControlChart.calculateDataAndRenderChart();

    // register a handler for timeframe being changed
    AJS.$(GH.RapidBoard.ChartView).unbind("timeFrameChanged").bind("timeFrameChanged", GH.RapidBoard.ChartView.ControlChart.handleTimeFrameChanged);
};

/**
 * Calculates the chart data and then renders the chart. This function needs to be called whenever the filters change
 */
GH.RapidBoard.ChartView.ControlChart.calculateDataAndRenderChart = function() {
	// calculate the cycle time data
	GH.ControlChart.Data.calculateCycleTimeData();
	
	// TODO: move to a more suitable place
	// initialize the time frames (as these depend on the preprocessed data)
	var minXValue = GH.ControlChart.Data.getMinXValue();
	var currentTime = GH.ControlChart.Data.getCurrentTime();
	GH.ChartTimeFrames.setBoundries(minXValue, currentTime);
	
	// render the chart now
	GH.RapidBoard.ChartView.ControlChart.renderChart();
};

/**
 * Renders the chart. Time frame information is applied on the fly before rendering.
 */
GH.RapidBoard.ChartView.ControlChart.renderChart = function() {
	// first filter regarding the time frame
	GH.ControlChart.Data.applyTimeWindow();

	// prepare the data
	GH.ControlChart.Data.prepareControlChartData();
	
    // render the chart statistics
    GH.RapidBoard.ChartView.ControlChart.renderChartStats();
	
	// fetch the series and stats
	var seriesData = GH.ControlChart.Data.getSeriesData();
	var stats = GH.ControlChart.Data.getStats();
	
	// element that holds the actual chart
    var plotHolder = AJS.$('#ghx-chart-view');
    
    // setup the chart    
    options = {
        xaxis: {
            mode: "time",
            min: stats.xMin,
            max: stats.xMax
        },
        series: {
            points: {
                show: true,
                radius: 3,
                symbol: "circle"
            }
        },
        grid: {
            hoverable: true,
            clickable: true
        },
        selection: {
        	mode: "x"
        },
        yaxis: {
            // enable compression of y axis
        	/*, min: chartData.minDuration, max: chartData.maxDuration*/
        	transform: GH.ControlChart.Data.compressAxis,
        	inverseTransform: GH.ControlChart.Data.inverseCompressAxis
        }
    };

    // fetch the series
    var series = GH.RapidBoard.ChartView.ControlChart.getChartSeries(seriesData, stats);

    // set the chart labels
    GH.RapidBoard.ChartView.setAndAlignAxisLabels(
    	AJS.I18n.getText('gh.rapid.charts.issuecompletiondate'),
    	AJS.I18n.getText('gh.rapid.chart.controlchart.days')
    );
    
    // plot the series
    GH.Chart.draw(plotHolder, series, options);

    // register plothover and plotclick event handlers
    // we first unbind here, otherwise we add on each invocation a new click handler
    plotHolder.unbind("plothover plotclick plotselected dblclick");
    plotHolder.bind("plothover", GH.RapidBoard.ChartView.ControlChart.handlePlotHover);
    plotHolder.bind("plotclick", GH.RapidBoard.ChartView.ControlChart.handlePlotClick);
    plotHolder.bind("plotselected", GH.RapidBoard.ChartView.ControlChart.handleRangeSelection);
    plotHolder.bind("dblclick", GH.RapidBoard.ChartView.ControlChart.resetRangeSelection);
};

GH.RapidBoard.ChartView.ControlChart.handleTimeFrameChanged = function() {
	GH.RapidBoard.ChartView.ControlChart.renderChart();
};

/**
 * Puts together the series to display
 * seriesData contains following elements:
 * {
 *    data: [], // actual line
 *    aggregatedMean: [],
 *    mean: [], // 2 values to draw a line
 *    deviations[][] // array of deviation lines
 * }
 */
GH.RapidBoard.ChartView.ControlChart.getChartSeries = function(seriesData, stats) {
    // put together the series we actually display
    var series = [];
    
    var seriesName = AJS.I18n.getText('gh.rapid.chart.controlchart.cycletime')
    
    // actual data points
    if (seriesData.values) {
        // actual point values
        series.push({
            data: seriesData.values,
            label: seriesName
        });
    }
    
    // mean line
    if (seriesData.mean) {
    	series.push({
            id: 'mean',
            label: AJS.I18n.getText('gh.rapid.chart.dataset.meananddeviation.legend', seriesName),
            data: seriesData.mean,
            points: { show:false },
            lines: { show: true },
            color: "#4267b1",
            shadowSize: 0,
            clickable: false,
            hoverable: false
        });    	
    }
    
    // aggregated mean
    if (seriesData.aggregatedMean) {
        series.push({
        	id: 'aggregateMean',
        	label: AJS.I18n.getText('gh.rapid.chart.dataset.aggregatedmean.legend', seriesName, stats.windowLengthInDays),
            data: seriesData.aggregatedMean,
            points: { show:false },
            lines: { show: true },
            shadowSize: 0,
            clickable: false,
            hoverable: false,
            color: "#999"
        });
    }
     
    // deviations
    if (seriesData.deviations) {
    	for (var i=0; i < seriesData.deviations.length; i++) {
    		var deviation = seriesData.deviations[i];
    		series.push({
	            data: deviation,
	            points: { show:false },
	            lines: {
	                show: true,
	                lineWidth:0.08,
	                fill: 0.03
	            },
	            color: "#9eb6d4",
	            fillBetween: 'mean',
	            clickable: false,
	            hoverable: false
    		});
    	}
	}
    
    return series;
};

/**
 * Renders the data part for the chart
 */
GH.RapidBoard.ChartView.ControlChart.renderChartStats = function() {
	// fetch the stats
	var stats = GH.ControlChart.Data.getStats();
	
    // render the info
    var dataDiv = AJS.$(GH.tpl.rapid.chart.renderControlChartDetails({stats:stats}));
    AJS.$('#ghx-chart-data').empty().append(dataDiv);
};

/**
 * Called when a range selection has been made
 */
GH.RapidBoard.ChartView.ControlChart.handleRangeSelection = function(event, ranges) {
	// set to custom and use from and to
	GH.ChartTimeFrames.setNewChartLimits(-1, ranges.xaxis.from, ranges.xaxis.to, true);
};

/**
 * Called on a double click to reset the selection
 */
GH.RapidBoard.ChartView.ControlChart.resetRangeSelection = function(event) {
	// set to All Time
	GH.ChartTimeFrames.setNewChartLimits(0, 0, 0, true);
};

/**
 * Handler for plot click events
 */
GH.RapidBoard.ChartView.ControlChart.handlePlotClick = function (event, pos, item) {
    if (item) {
    	var issueData = GH.ControlChart.Data.getIssueAtIndex(item.dataIndex);
        var issueKey = issueData && issueData.issue.key;
        if (issueKey) {
            // check whether we have the data for that issue in our issue view
            var issueData = GH.RapidBoard.RapidListDataController.getIssueDataByKey(issueKey);
            if (issueData && !issueData.isFiltered) {
                var issueKey = issueData.key;
                // TODO: cleanup!
                GH.RapidBoard.State.setSelectedIssueKey(null);
                GH.RapidBoard.State.setDetailsViewOpened(false);
                GH.RapidBoard.ChartView.closeChartView(null, true);
                GH.RapidBoard.DetailsView.showIssueDetails(issueKey);
            }
        }
    }
};

/**
 * Handler for  plot hover events.
 */
GH.RapidBoard.ChartView.ControlChart.handlePlotHover = function(event, pos, item) {
	if (item) {
		// fetch the issue for that index
		var issueData = GH.ControlChart.Data.getIssueAtIndex(item.dataIndex);
        if (GH.RapidBoard.ChartView.ControlChart.previousPlotHoverPoint != item.dataIndex) {
            GH.RapidBoard.ChartView.ControlChart.previousPlotHoverPoint = item.dataIndex;
            
            // remove the previous tooltip and add a new one
            AJS.$("#ghx-tooltip").remove();
            GH.RapidBoard.ChartView.ControlChart.showChartTooltip(item, issueData);
            
            // if the tooltip has no warnings then the issue is clickable
            if(!AJS.$("#ghx-tooltip").hasClass("ghx-has-warnings")) {
                AJS.$("#ghx-chart-view").addClass("ghx-clickable");
            }
        }
    }
    else {
        AJS.$("#ghx-tooltip").remove();
        GH.RapidBoard.ChartView.ControlChart.previousPlotHoverPoint = null;
        AJS.$("#ghx-chart-view").removeClass('ghx-clickable');
    }
};
/** Holds the last hovered point. */
GH.RapidBoard.ChartView.ControlChart.previousPlotHoverPoint = undefined;

GH.RapidBoard.ChartView.ControlChart.roundDuration = function(duration) {
	return duration != 0 ? Math.round(duration*10) / 10 : 0;
};

/**
 * Displays a data point tooltip
 */
GH.RapidBoard.ChartView.ControlChart.showChartTooltip = function(item, issueData) {
    var x = item.pageX;
    var y = item.pageY;
    var series = item.series;
    var dataIndex = item.dataIndex;
    var seriesLabel = series.label;
    var issueKey = issueData.issue.key;
    var summary = issueData.issue.summary;
    var duration = item.datapoint[1];
    duration = GH.RapidBoard.ChartView.ControlChart.roundDuration(duration); 

    // fetch the issue object in the pool
    var boardIssue = GH.RapidBoard.RapidListDataController.getIssueDataByKey(issueKey);
    
    var tooltip = AJS.$('<div id="ghx-tooltip"><strong>' + issueKey + ' </strong></div>').css( {
        position: 'absolute',
        display: 'none',
        top: y + 5,
        //left: x + 15,
        'border-left-color': series.color
    });
    if(summary)
    {
        tooltip.append('<div class="ghx-summary">' + soy.$$escapeHtml(summary) + '</div>');
    }
    
    // append information about the individual columns
    var activeColumnIds = GH.ChartFilters.getActiveColumnIds();
    if (activeColumnIds.length > 1) {
    	// holds the columns
    	var colStats = AJS.$('<ul class="ghx-stats"></ul>');
    	tooltip.append(colStats);
    	
    	var columns = GH.ChartFilters.getColumns();
    	for(var i=0; i < columns.length; i++) {
    		var column = columns[i];
    		if (column.active) {
    			var durationInDays = GH.ChartTimeFrames.millisToDays(issueData.issue.durations[i]);
    			durationInDays = GH.RapidBoard.ChartView.ControlChart.roundDuration(durationInDays);
    			var columnDays = AJS.I18n.getText("gh.rapid.chart.numberofdays", durationInDays);
    			colStats.append(AJS.$('<li><em>' + soy.$$escapeHtml(column.name) + ':</em><strong>' + columnDays + '</strong></li>'))
    		}
    	}
        // total
        var daysInfo = AJS.I18n.getText("gh.rapid.chart.numberofdays", duration);
        colStats.append(AJS.$('<li><em>' + soy.$$escapeHtml(seriesLabel) + ':</em><strong>'+ daysInfo + '</strong></li>'));
    }
    
    if(!boardIssue)
    {
        tooltip.addClass("ghx-has-warnings").append('<div class="aui-message warning"><span class="aui-icon icon-warning"></span>' + AJS.I18n.getText("gh.rapid.charts.issue.excess")+'</div>')
    } else if(boardIssue.isFiltered)
    {
        tooltip.addClass("ghx-has-warnings").append('<div class="aui-message warning"><span class="aui-icon icon-warning"></span>' + AJS.I18n.getText("gh.rapid.charts.issue.filtered")+'</div>')
    }

    
    // append to the body (which will give us the correct with)
    tooltip.appendTo("body");

    // check whether we overshoot the right border, in which case we want to display the tooltip to the left
    var tooltipWidth = tooltip.width();
    var canvas = AJS.$('#ghx-chart-view canvas.base'); 
    
    // now set the correct x position for the tooltip
    if (x + tooltipWidth > canvas.offset().left + canvas.width()) {
        tooltip.css({ left: x - (30 + tooltipWidth)});
    } else {
        tooltip.css({ left: x + 15});
    }
    tooltip.fadeIn(200);
};
