/**
 * Controls the data for the control chart.
 */
if (!GH.ControlChart) GH.ControlChart = {};
GH.ControlChart.Data = {};

/**
 * Holds the response data object
 */
GH.ControlChart.Data.rawData = {};

/**
 * Holds the current cycle time data, unfiltered
 * Each entry object represents an issue, its done time and cycle time filtered according to the currently selected columns.
 */
GH.ControlChart.Data.currentData = [];
GH.ControlChart.Data.minXValue = 0;

/**
 * Holds the current cycle time data filter by the selected time frame
 */
GH.ControlChart.Data.currentDataFiltered = [];

/**
 * Holds the current statistics
 */
GH.ControlChart.Data.stats = {};

/**
 * The calculated series data
 */
GH.ControlChart.Data.seriesData = [];

/**
 * Sets the control chart data received from the server
 */
GH.ControlChart.Data.setRawData = function(data) {
	// set the supplied data object
	GH.ControlChart.Data.rawData = data;
};

/**
 * Calculates the current data objects
 */
GH.ControlChart.Data.calculateCycleTimeData = function() {
    var currentData = [];
    var rawData = GH.ControlChart.Data.rawData;
    
    // fetch the active column indexes, as we ignore all other columns
    var columns = GH.ChartFilters.getColumns(); 
    var activeColumnIndexes = [];
    for (var i=0; i < columns.length; i++) {
    	if (columns[i].active) {
    		activeColumnIndexes.push(i);
    	}
	}
    
    // fetch "now"
    var now = rawData.currentTime;
    
    outerloop:
    for (var x=0; x < rawData.issues.length; x++) {
        var issue = rawData.issues[x];
        
        // find the maximum end date for the issue (thus the issue passes at least one of the columns)
        var endDate = -1;
        var totalDuration = 0;
        for (var i=0; i < activeColumnIndexes.length; i++) {
        	// fetch leaveTime and duration
        	var leaveTime = issue.leaveTimes[activeColumnIndexes[i]];
        	var duration = issue.durations[activeColumnIndexes[i]];
        	
        	// skip issue if it is still in the column
        	if (leaveTime == now) {
        		// skip issue
        		continue outerloop;
        	}
        	
        	// add the duration, if the column is non-visited this will be 0.
        	totalDuration += duration;
        	
        	// we take the latest leave time, use if newer than previous endDate
        	if (leaveTime > endDate) {
        		 endDate = leaveTime;
        	}
        }
        
        // skip if the issue didn't pass through
        if (endDate < 0) {
        	continue;
        }
        
        // add the data point
        currentData.push({
        	issue: issue,
        	endDate: endDate,
        	duration: totalDuration
        });
    }

    // sort the data according to endDate, otherwise all other calculations will fail
    currentData = _.sortBy(currentData, function(entry) { return entry.endDate; });
    
    // done filtering the issues
    GH.ControlChart.Data.currentData = currentData;

    
    // now also calculate the minXValue in this data.
    if (currentData.length > 0) {
    	GH.ControlChart.Data.minXValue = currentData[0].endDate;
	} else {
		GH.ControlChart.Data.minXValue = null;
	}
};

/**
 * Get the current time according to the chart data
 */
GH.ControlChart.Data.getCurrentTime = function() {
	return GH.ControlChart.Data.rawData.currentTime;
};

/**
 * Returns the first x value of the current data or null if no data exists
 */
GH.ControlChart.Data.getMinXValue = function() {
	return GH.ControlChart.Data.minXValue;
};

/** 
 * Reduces the data to the specified time frame 
 */
GH.ControlChart.Data.applyTimeWindow = function() {
	// set an empty array if we got no data
	var currentData = GH.ControlChart.Data.currentData;
	if (currentData.length == 0) {
		GH.ControlChart.Data.currentDataFiltered = currentData;
	}
	
	// fetch the current min and max
	var xMin = GH.ChartTimeFrames.getChartStartDate(); //GH.ControlChart.Data.getXMin();
	var xMax = GH.ChartTimeFrames.getChartEndDate(); //GH.ControlChart.Data.getXMax();
	
	// filter the issues regarding endDate, remove all that don't fall into the 
	var filtered = [];
	for(var i = 0; i < currentData.length; i++) {
		if(xMin <= currentData[i].endDate && currentData[i].endDate <= xMax) {
			filtered.push(currentData[i]);
		}
	}
	GH.ControlChart.Data.currentDataFiltered = filtered;
};

/**
 * Transforms the internal data (issue, endDate, duration) into a serie of x/y points.
 * 
 * This serie is then used as basis for calculating all stats as well as displayed in the chart.
 * All duration values are transformed to days, with the result that all stats will be days based as well
 */
GH.ControlChart.Data.convertDataToSerie = function(data) {
	var values = [];
	for (var i=0; i < data.length; i++) {
		var entry = data[i];
		// the duration is in milliseconds, the chart displays it in days though so transform here
		var duration = GH.ControlChart.Data.millisToDays(entry.duration);
		values.push([entry.endDate, duration]);
	}
	return values;
};

/**
 * Get the chart series data. The returned object contains several series:
 * {
 *    data: [], // actual line
 *    aggregatedMean: [],
 *    mean: [], // 2 values to draw a line
 *    deviations[][] // array of deviation lines
 * }
 */
GH.ControlChart.Data.prepareControlChartData = function() {
	var seriesData = {};
	
	// calculate the actual points, with y values being already transformed to days
	// as input serves the currentDataFiltered, thus the data already stripped by values outside the time frame
	seriesData.values = GH.ControlChart.Data.convertDataToSerie(GH.ControlChart.Data.currentDataFiltered);
	
	// calculate the stats based on these values
	GH.ControlChart.Data.calculateStats(seriesData.values);
	var stats = GH.ControlChart.Data.stats;
	
	// put together the mean line
	seriesData.mean = [[stats.xMin, stats.mean],[stats.xMax, stats.mean]];
	
	// calculate the aggregated mean
    GH.ControlChart.Data.calculateAggregatedMean(seriesData.values, stats.xMin, stats.xMax, stats.windowLength, stats.windowShiftDelta, seriesData);
	
    // only add deviation lines if we got more than 1 issue 
    if (seriesData.values.length > 1) {
	    // add the deviation lines
	    var deviations = [];
	    for (var i=1; i<10; i++)
	    {
	    	// lower line
	    	var y = stats.mean - i*stats.deviation;
	    	y = y < stats.min ? stats.min : y; // TODO: why are we bound by min and max?
	    	deviations.push([[stats.xMin, y],[stats.xMax, y]]);
	    	
	    	var y = stats.mean + i*stats.deviation;
	    	y = y > stats.max ? stats.max : y;
	    	deviations.push([[stats.xMin, y],[stats.xMax, y]]);
	    }
	    seriesData.deviations = deviations;
    }
	    
    GH.ControlChart.Data.seriesData = seriesData;
};



GH.ControlChart.Data.getSeriesData = function() {
	return GH.ControlChart.Data.seriesData;
};

GH.ControlChart.Data.getStats = function() {
	return GH.ControlChart.Data.stats;
};

GH.ControlChart.Data.getIssueAtIndex = function(dataIndex) {
	return GH.ControlChart.Data.currentDataFiltered[dataIndex];
};


/**
 * Calculates the different stats
 */
GH.ControlChart.Data.calculateStats = function(data) { 
	var stats = {};
	
	// calculate min, max, mean, store result inside serie
	GH.ControlChart.Data.calculateMinMaxMean(data, stats);
	
	// calculate deviation
	GH.ControlChart.Data.calculateDeviation(data, stats.mean, stats);
	
	// fetch the xMin/xMax we look at
	var xMin = GH.ChartTimeFrames.getChartStartDate();
	var xMax = GH.ChartTimeFrames.getChartEndDate();
	stats.xMin = xMin;
	stats.xMax = xMax;
    
    // calculate the params required for the mean aggregation
    GH.ControlChart.Data.calculateAggregatedMeanParams(xMin, xMax, stats);

    GH.ControlChart.Data.stats = stats;
};

/**
 * Calculates the min and max values as well as the mean over all values.
 * The result values are stored in resultObj.
 */
GH.ControlChart.Data.calculateMinMaxMean = function(data, resultObj) {
    // handle case without data
    if (data.length < 1) {
    	resultObj.min = 0;
    	resultObj.max = 0;
    	resultObj.mean = 0;
    }
    else {
	    // One loop to get min, max, sum and then mean
	    var min = Number.MAX_VALUE;
	    var max = Number.MIN_VALUE;
	    var sum = 0;
	    for(var i=0; i < data.length; i++) {
	        sum += data[i][1];
	        if(data[i][1] < min) {
	            min = data[i][1];
	        }
	        if(data[i][1] > max) {
	            max = data[i][1];
	        }
	    }
	    resultObj.min = min;
	    resultObj.max = max;
	    resultObj.mean = sum / data.length;
    }
};

/**
 * Calculates the deviation for a given set of data and mean
 * The deviation is defined as the sqrt(sum((value-mean)^2) / (total-1)),
 * @return the deviation
 */
GH.ControlChart.Data.calculateDeviation = function(data, mean, resultObj) {
    // deviation is 0 in case of 0 or 1 values
    if (data.length < 2) {
    	resultObj.deviation = 0;
    }
    else {
    	// the deviation is the 
	    var deviationSum = 0;
	    for (var i=0; i < data.length; i++) {
	        var pointDeviation = mean - data[i][1];
	        deviationSum += Math.pow(pointDeviation, 2);
	
	    }
	    var division = deviationSum / (data.length - 1);
	    resultObj.deviation = Math.sqrt(division);
    }
};

/**
 * Calculates the aggregated mean series for a given set of values
 *
 */
GH.ControlChart.Data.calculateAggregatedMeanParams = function(min, max, resultObj) {
    // default values for window length and shift delta
    var day = 24*60 * 60 * 1000;
    var windowLength = 30 * day; // width of a window
    var windowShiftDelta = 15 * day; // shift of the window
    
    // for now we try to come up with a good default for windowLength, and use the length as shift delta
    var numDays = (max - min) / day;
    if (numDays < 14) { // less than 14 days / 2 weeks
    	windowLength = windowShiftDelta = 1*day;
    } else if (numDays < 30) { // less than 30 days/4 weeks
    	windowLength = windowShiftDelta = 2*day;
    } else if (numDays < 90) { // less than 90 days/3 months
    	windowLength = windowShiftDelta = 7*day;
    } else if (numDays < 180) { // less than 180 days / 6 months
    	windowLength = windowShiftDelta = 14*day;
    } else { // more than 6 months
    	windowLength = windowShiftDelta = 30*day;
    }
    // use shift length as delta
    //windowShiftDelta = windowLength;
    windowShiftDelta = 1*day;
    
    // set the window length, delta and length in days
	resultObj.windowLength = windowLength;
	resultObj.windowShiftDelta = windowShiftDelta;
	resultObj.windowLengthInDays = windowLength / day;
};

/**
 * Calculates the aggregated mean series given the data, min and max (X axis), the window length and shiftDelta
 */
GH.ControlChart.Data.calculateAggregatedMean = function(data, min, max, windowLength, windowShiftDelta, resultObj) {
	// safety check
	if (windowLength <= 0) return [];
	
	// we calculate through all windows, thus even if the chart is empty we will get a value 
    var tmpSum = 0,
    	tmpCount = 0,
    	windowStart = min - windowLength, // start outside and slowly move in
    	windowEnd = windowStart + windowLength,
    	firstEntryIndex = 0,
    	aggregatedMean = [];
    
    while(windowStart < max) {
    	// calculate the x point for this window
    	var xPoint = windowStart + windowLength / 2;
    	
    	// skip if we are before the range
    	if (xPoint >= min) {
	    	
	    	// calculate the window
	    	for(var i=firstEntryIndex; i < data.length; i++) {
	    		var currX = data[i][0];
	    		if (currX < windowStart) {
	    			firstEntryIndex = i+1; // skip from now on
	    		}
	    		else if (currX < windowEnd) {
	    			// count
	    			tmpSum += data[i][1];
	        		tmpCount += 1;
	    		}
	    		else {
	    			// we passed the window, stop here
	    			break;
	    		}
	    	}
	    	
	    	// calculate the mean
			var windowMean = tmpCount > 0 ? tmpSum / tmpCount : 0;
			
			// add the point...
			// if this is the first one, also add one exactly at min
			if (aggregatedMean.length == 0 && xPoint > 0) {
				aggregatedMean.push([min, windowMean]);
			}
		
			// add the actual point
			aggregatedMean.push([xPoint, windowMean]);
    	}
		
		// move window and reset counters
		windowStart += windowShiftDelta;
		windowEnd = windowStart + windowLength;
		tmpSum = tmpCount = 0;
		
    	// if are past the range, make sure we have a point at max
		if (xPoint + windowShiftDelta > max && xPoint != max) {
			aggregatedMean.push([max, windowMean]);
			break;
		}
    }
        
    resultObj.aggregatedMean = aggregatedMean;
};

/**
 * Transforms the duration to a more meaningful value, e.g. days
 */
GH.ControlChart.Data.millisToDays = function(duration) {
    if (duration) {
        return duration / (1000 * 60 * 60 * 24);
    } else {
        return duration;
    }
};

GH.ControlChart.Data.daysToMillis = function(days) {
    if (days) {
        return days * (1000 * 60 * 60 * 24);
    } else {
        return days;
    }
};


/**
 * Compress the axis, uses the standard deviation. i.e. if you are in the third sd the number is divided by 3
 */
GH.ControlChart.Data.compressAxis = function(v) {
	var mean = GH.ControlChart.Data.stats.mean;
	var sd = GH.ControlChart.Data.stats.deviation;
    
    if (v == mean || v < 0 || sd == 0)
        return v;
    
    var distanceMean = Math.abs(v-mean);
    var currentSD = Math.ceil(distanceMean/sd);
    var distanceSD;
    
    var sum;
    if (v-mean >= 0) {
        sum = mean;
        distanceSD = Math.abs(v - mean - (currentSD - 1)*sd);
    }
    else {
        sum = -mean;
        distanceSD = Math.abs(v - (mean - (currentSD - 1)*sd));
    }

    for (var i = 2; i<=currentSD; i++) {
        sum = sum + sd/(i-1);
    }

    return Math.abs(distanceSD/currentSD + sum);
}

GH.ControlChart.Data.inverseCompressAxis = function(v) {
    // For some reason if this isnt defined the hover doesnt work, but the function does nothing.
}
